# frozen_string_literal: true
# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class EntitiesController < Hermes::Api::Dmo::ApplicationController
        before_action :filter_params, only: :update

        class << self
          def mutate_params
            {
              achieved_goal: { type: 'Boolean' },
              auto_request_validation: { type: 'Boolean' },
              auto_validation: { type: 'Boolean' },
              id_cristal: { type: 'String' },
              id_rex: { type: 'String' },
              in_peec: { type: 'Boolean' },
              lessor_id: { type: 'String' },
              priorities_enabled: { type: 'Boolean' },
              siren: { type: 'String' },
              siret: { type: 'String' },
              title: { type: 'String' },
            }
          end 
      
          def acceptable_params
            {
              annual_stock: { type: 'Integer' },
              available_points: { type: 'Integer' },
              group_id: { type: 'String' },
              is_root: { type: 'String' },
              manager_group_id: { type: 'String' },
              parent_id: { type: 'String' },
              root_id: { type: 'String' },
              spent_on_completed_requests: { type: 'Integer' },
              spent_on_ongoing_requests: { type: 'Integer' },
              spent_on_other: { type: 'Integer' },
              supervisor_group_id: { type: 'String' },
              total_spent: { type: 'Integer' },
              cre_national_id_cristal: { type: 'String'},
              cre_national_email: { type: 'String'},
              cre_national_first_name: { type: 'String'},
              cre_national_last_name: { type: 'String'},
            }.merge(self.mutate_params).merge(super)
          end 
      
          def valid_operators
            %w[$in $nin $ne $gt $gte $lt $lte $eq $regex]
          end
        end
        
        def can_see_entities
          !(current_user_roles & ['Administrator', 'superadministrateur']).empty?
        end 

        def show 
          return render_json_error(:unauthorized, :cannot_see_entities) unless can_see_entities

          super 
        end 

        def index
          return render_json_error(:unauthorized, :cannot_see_entities) unless can_see_entities

          acceptable_params *self.class.acceptable_params.keys
          valid_operators *self.class.valid_operators
          @query_precision ||= {}
  
          paginate(
            index_aggregation(no_acl: true),
            each_serializer: @serializer,
            **@options
          )
        end
          
        def admin_search
          return render json: {status: "unauthorized", code: 401, message: "user is no superadmin"} unless is_super_admin

          aggregation = Hermes::UserRight.all

          if !params['entity_params'].blank?
            entity_query = admin_search_param(
              params: params['entity_params'], 
              possible_params: [:id, :id_cristal, :siren, :siret],
              partial_regex_params: [:title]
            )
            entity_hint = entity_query["title"] ? 'title_1' : nil
            aggregation = aggregation.where(:entity_id.in => Hermes::Entity.where(entity_query).hint(entity_hint).pluck(:id)) 
          end 

          if !params['user_params'].blank?
            user_query = admin_search_param(
              params: params['user_params'], 
              possible_params: [:id, :email, :auth_roles],
              full_regex_params: [:first_name, :last_name]
              )
            user_hint = nil
            user_hint = 'first_name_1' if user_query["first_name"]
            user_hint = 'last_name_1' if user_query["last_name"]
            aggregation = aggregation.where(:user_id.in => User.where(user_query).hint(user_hint).pluck(:id)) 
          end 

          @options[:no_attributes] = true

          paginate(
            aggregation,
            each_serializer: Hermes::AdminSearchRightSerializer,
            **@options
          )
        end
        
        def update_annual_stock
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_entity, :annual_stock_param_missing) unless params[:annual_stock].present?
          # return render_json_error(:unauthorized, :not_authorized) unless @instance.writable_by?(current_user)
          return respond_with_errors(@instance) unless @instance.update(annual_stock: params[:annual_stock])

          render(
            jsonapi: @instance,
            serializer: @serializer,
            **@options
          )
        end 

        def can_create_tier_account
          Hermes::Entity.readable_by(current_user).find_by(id: @instance.id) && 
          (
            current_user.auth_roles&.include?('entreprise') || 
            current_user.auth_roles&.include?('entreprise administrateur')
          )
        end

        def checks_before_create_account
          @instance = Hermes::Entity.find_by(id: params[:id]) if params[:id]
          return [:not_found, :instance_not_found] unless @instance
          return [:unauthorized, :no_right_on_this_entity] if !can_create_tier_account
          if [:id, :first_name, :last_name, :email, :gender, :phone_number].any? { |p| !params.has_key?(p)  }
            return [:unprocessable_entity, :user_information_missing] 
          end
        end 


        def switch_manual_mode
          return render_json_error(:unauthorized, :unauthorized) unless is_super_admin
          return render_json_error(:not_found, :instance_not_found) unless entity = Hermes::Entity.find(params[:id])
          entity.switch_manual_mode            
          render json: {status: "success", code: 200, message: "manual mode switch to #{entity.manual_mode}"}
        end 

        def generate_user_right_data(users, type)
          users.map { |u| {user_id: u.id.to_s, email: u.email, first_name: u.first_name, last_name: u.last_name, phone_number: u.phone_number, auth_roles: u.auth_roles, right: type}} 
        end

        def user_rights
          return render_json_error(:not_found, :instance_not_found) unless @instance

          if params[:user_id]
            user = User.find(params[:user_id])
            return render_json_error(:not_found, :user_not_found) unless user
            
            right = nil
            
            user.group_ids.each do |id|
              right = 'reader' if id == @instance.group_id
              right = 'user' if id == @instance.supervisor_group_id
              right = 'administrator' if id == @instance.manager_group_id
            end
            
            data = {}
            if right
              data = {
                user_id: user.id.to_s, 
                email: user.email, 
                first_name: user.first_name, 
                last_name: user.last_name, 
                phone_number: user.phone_number, 
                auth_roles: user.auth_roles, 
                right: right
              }
            end
          else
            data = []
            # Fix temporaire avant spération des entity/lessor
            exclude_auth_roles = ['cre', 'rdsl']
  
            readers = User.where(group_ids: @instance.group_id, auth_roles: {"$nin": exclude_auth_roles})          
            users = User.where(group_ids: @instance.supervisor_group_id, auth_roles: {"$nin": exclude_auth_roles})          
            administrator = User.where(group_ids: @instance.manager_group_id, auth_roles: {"$nin": exclude_auth_roles})
            
            data = data + generate_user_right_data(readers, 'reader')
            data = data + generate_user_right_data(users, 'user')
            data = data + generate_user_right_data(administrator, 'administrator')
          end

          render json: data
        end

        def filter_params
          private_fields = %w[auto_validation auto_request_validation]

          if (params[:data][:attributes].keys - private_fields).any? 
            unless @instance && @instance.writable_by?(@user)
              private_fields.each { |field|  params[:data][:attributes].delete field }
            end
          end
        end

        def housing_requests_count
          return render_json_error(:not_found, :instance_not_found) unless @instance

          in_process = Hermes::HousingRequest.where(
            is_active: true,
            entity_id: @instance.id, 
            entity_status: { :$in => ['EN COURS DE SAISIE'] },
            consent_level: 4
          ).count

          delegations = Hermes::HousingRequest.where(
            is_active: true,
            entity_id: @instance.id, 
            entity_status: { :$in => ["A VALIDER", "EN COURS", "EN COURS DE SAISIE"] },
            consent_level: 4
          ).count


          render json: {
            delegations: delegations,
            in_process: in_process
          }
        end
      end
    end
  end
end
