# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des methodes
module Hermes
  module Api
    module Dmo
      class HousingApplicationsController < Hermes::Api::Dmo::ApplicationController
        require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s

        class << self
          def mutate_params
            {
              housing_request_id: { type: 'String'},
              housing_offer_id: { type: 'String'},
              epci: { type: 'Integer'},
              max_date: { type: 'DateTime'},
              status: { type: 'String'},
              visit_status: { type: 'String'},
              visit_conducted_date: { type: 'DateTime'},
              lessor_event_informations: { type: 'String'},
              last_status_update_date: { type: 'Date'},
              rank: { type: 'Integer'},
              date_event: { type: 'Date'},
              motive_code: { type: 'String'},
            }
          end 
      
          def acceptable_params
            {
              applicated_at: { type: DateTime },
              status_updated_at: { type: DateTime },
              visit_conducted_date: { type: DateTime },
              visit_status_updated_at: { type: DateTime },
              profile_score: { type: Integer },
              eligible_position: { type: Integer },
              applicated_position: { type: Integer },
              is_active: { type: Boolean },
              als_id: { type: String },
              cotations: { type: Hash },
              business_cotation: { type: Float },
              applicant_used_for_siret_firstname: { type: String },
              applicant_used_for_siret_lastname: { type: String },
              applicant_used_for_siret_email: { type: String },
              applicant_used_for_siret_birth_date: { type: Date },
            }.merge(self.mutate_params).merge(super)
          end 
      
          def valid_operators
            %w[$in $nin $ne $gt $gte $lt $lte $regex]
          end

          def regex_params
            {}
          end 
        end
        
        def index
          acceptable_params *self.class.acceptable_params.keys
          valid_operators *self.class.valid_operators

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}

          aggregation = index_aggregation

          if params[:unique_numbers]
            aggregation = aggregation.where(:housing_request_id.in => Hermes::HousingRequest.where(unique_numbers: params[:unique_numbers]).pluck(:id))
          end 

          options[:join_housing_offer]= true if params[:join_housing_offer]
          options[:lignt_serializer] = true 
          
          paginate(
            aggregation,
            each_serializer: Hermes::HousingApplicationSerializer,
            **options
          )
        end

        def show
          @options[:join_housing_offer]= true if params[:join_housing_offer]
          super
        end

        def create
          create_without_render
          @instance.skip_check = true if params[:options]&.include?('skip_check')
      
          return render_json_error(:unprocessable_entity, :no_right_to_change_status) unless can_update_status
          return respond_with_errors(@instance) unless @instance.save
      
          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end

        def update
          return render_json_error(:unprocessable_entity, :no_right_to_change_status) unless can_update_status
          super
        end 

        def update_status
          return render_json_error(:unprocessable_entity, :missing_status) unless params["data"]["attributes"]["status"]
          return render_json_error(:unprocessable_entity, :missing_unique_number) unless params["data"]["attributes"]["unique_number"]
          return render_json_error(:unprocessable_entity, :missing_external_ref) unless params["data"]["attributes"]["external_ref"]

          hr = Hermes::HousingRequest.find_by(
            is_active: true, 
            :request_status.nin => ['ANNULEE', 'SATISFAITE'], 
            unique_numbers: params["data"]["attributes"]["unique_number"]
          )
          return render_json_error(:unprocessable_entity, :housing_request_not_found) unless hr

          @instance = hr.active_application
          return render_json_error(:unprocessable_entity, :housing_application_not_found) unless @instance
          return render_json_error(:unprocessable_entity, :external_ref_not_matching_application) unless @instance.housing_offer.external_ref == params["data"]["attributes"]["external_ref"]

          unless @instance.reload && @instance.writable_by?(@user)
            Hermes::LogJob.add_to_queue(level: 'error', housing_application_id: @instance.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status -- User not writer", data: params["data"]["attributes"].as_json );
            return render_json_error(:unprocessable_entity, :not_writers)
          end
          return render_json_error(:unprocessable_entity, :no_right_to_change_status) unless can_update_status
          
          old_status = @instance.status&.to_s
          new_status = params["data"]["attributes"]["status"]&.to_s

          allowed_old_statuses = Hermes::Statuses::HousingApplication.possible_status.values.flatten;
          allowed_new_statuses = [
            "INSCRIPTION CAL",
            "RETENUE EN CAL SOUS RESERVE",
            "RETENUE EN CAL",
            "RETENUE EN CAL SOUS CONDITIONS",
            "REFUSEE EN CAL",
            "DESISTEMENT",
            "VISITE PROPOSEE",
            "VISITE EFFECTUEE",
            "LOGE",
            "ABANDON BAILLEUR",
            "ANNULEE",
            "EN ATTENTE",
            "DESISTEMENT AVANT CAL",
          ]

          not_valid_change = new_status && old_status && allowed_new_statuses.include?(new_status) && allowed_old_statuses.include?(old_status)
          unless not_valid_change
            Hermes::LogJob.add_to_queue(level: 'error', housing_application_id: @instance.id.to_s, code: "UPDATE_STATUS_ERR", action: "update_status -- cannot change #{old_status} to #{new_status}", data: params["data"]["attributes"].as_json )
            return render_json_error(:unprocessable_entity, :cannot_change_to_status_from_current_status)
          end

          @instance.force_update_status_task_for = 'status'
          if Hermes::Statuses::HousingApplication.possible_status[:visit_status].include?(params["data"]["attributes"]["status"])
            params["data"]["attributes"]["visit_status"] = params["data"]["attributes"]["status"]
            params["data"]["attributes"].delete("status")
            @instance.force_update_status_task_for = 'visit_status'
          end
          
          params["data"]["attributes"].delete("unique_number")
          params["data"]["attributes"].delete("external_ref")
          check_params_and_update
        end

        def attribution
          # https://www.wrike.com/open.htm?id=461622137
          @offer = Hermes::HousingOffer.find mutate_valid_params[:housing_offer_id]
          return render_json_error(:not_found, :object_not_found) unless @offer
          return render_json_error(:unprocessable_entity, :not_requalified) unless @offer.requalification

          if applicant_id = params['data']['attributes']['applicant_id']
            @applicant = Hermes::Applicant.find applicant_id
          elsif uniq_numb = params['data']['attributes']['nud']
            birth_date = params['data']['attributes']['birth_date']
            @applicant = Hermes::Applicant.find_or_create_by_nud(current_user, uniq_numb, birth_date)
          end


          if @applicant.errors.present?
            return render_json_error(:unprocessable_entity, :invalid_birth_date) if @applicant.errors.messages[:birth_date]
            return render_json_error(:not_found, :invalid_nud) if @applicant.errors.messages[:nud]
          end

          @applicant.housing_requests.each do |hr|
            hr.housing_applications.each do |ha|
              ha.update(status: 'ANNULEE', is_active: false)
            end # (a) si le demandeur a une candidature en cours elle est annulée
            hr.update(request_status: 'COMPLETE') # (b) passer les demandes en logé
          end

          @new_request = Hermes::HousingRequest.create(main_applicant: @applicant)
          @application = Hermes::HousingApplication.create(
            housing_offer: @offer,
            housing_request: @new_request,
            applicated_at: DateTime.current
          )
          @offer.update(lessor_status: 'LOUE', offer_status: "RETIRE PAR LE BAILLEUR") # (b) et l'offre en attribuée + (c) l'offre est comptabilisé dans les objectifs "attribués"
          @application.reload

          render(jsonapi: @application, serializer: @serializer, status: :created)
        end

        private

        def can_update_status
          params_status = params.dig(:data, :attributes, :status)
          allowed_status = ["EN ATTENTE", "ANNULEE", "DESISTEMENT AVANT CAL"]
          is_admin || allowed_status.include?(params_status) || params_status == @instance&.status && !@instance.new_record?
        end
      end
    end
  end
end
