# frozen_string_literal: true
require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::HousingOffersController, type: :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  

  check_mutate_params(
    model: Hermes::HousingOffer, 
    mutate_params: Hermes::Api::Dmo::HousingOffersController.mutate_params, 
    factory: :housing_offer,
    min_params: {
      longitude: 0.0,
      latitude: 0.0
    },
    not_checked_keys: [
      :latitude,
      :longitude,
    ]
  )

  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
  let!(:user) {create(:user)}
  let!(:als_rh) {Hermes::ReservationHolder.create_default}

  let!(:threshold_plus) do
    create(
      :threshold,
      type: "PLUS",
      codes: ["PLU"],
      thresholds_map: {
        "Paris et communes limitrophes"=>{"thresholds_list"=>[23721, 35452, 46473, 55486, 66017, 74286], "another_person"=>8278},
        "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[23721, 35452, 42616, 51046, 60429, 68001], "another_person"=>7577},
        "Autres régions"=>{"thresholds_list"=>[20623, 27540, 33119, 39982, 47035, 53008], "another_person"=>5912}
      }
    )
  end

  describe 'POST /housing_offers' do
    describe 'Create housing_offer with minimal valid datas' do
      before :each do
        authenticated_header(request, responsible)
        post 'create',
              data: {
                attributes: {offer_status: 'EN ATTENTE COMMERCIALISATION' , lessor_status: 'EN ATTENTE PUBLICATION'}
              }
      end
      it 'returns status code 201' do
        expect(response).to have_http_status(201)
        perform_enqueued_jobs do
          enqueued_jobs.each do |job|
            if job[:queue] == "exflux5job"
            end
          end
        end
      end
    end

    describe 'Create housing_offer on existing one' do
      let!(:lessor) {create(:lessor, user_id: user.id, title: 'Lessor', siren: "555555555")}
      let!(:older_housing_offer) do 
        create(
          :housing_offer, 
          user_id: user.id, 
          lessor_siren: "555555555", 
          auto_destroy_next_day: true,
          external_ref: "111", 
          typology: "T1", 
          offer_status: 'RETIRE PAR LE BAILLEUR', 
          lessor_status: 'RETIRE'
        )
      end
      let!(:housing_offer) do 
        create(
          :housing_offer, 
          user_id: user.id, 
          lessor_siren: "555555555", 
          auto_destroy_next_day: true,
          external_ref: "111", 
          typology: "T1"
        )
      end 

      let!(:data) do 
        {
          data: {
            attributes: {
              external_ref: "111",
              auto_destroy_next_day: true,
              typology: "T2",
              lessor_siren: "555555555",
              lessor_code: "000",
              address: '14 rue edgard quinet',
              postal_code: "30100",
            }
          }
        }
      end

      it 'updated existing one' do
        housing_offer.set(lessor_code: nil)
        authenticated_header(request, user)
        post 'create',data
        expect(response).to have_http_status(200)
        expect(json['data']['id']).to eq(housing_offer.id.to_s)
      end 
      

      it 'updated existing one with pictures' do
        picture1 = Hermes::Picture.create()
        picture2 = Hermes::Picture.create()
        housing_offer.update(ordered_picture_ids: [picture1.id.to_s, picture2.id.to_s])
        authenticated_header(request, user)
        expect(housing_offer.reload.ordered_picture_ids).to eq([picture1.id.to_s, picture2.id.to_s])
        post 'create',data
        expect(response).to have_http_status(200)
        expect(json['data']['attributes']["typology"]).to eq("T2")
        expect(json['data']['attributes']["lessor_code"]).to eq("000")
        
        expect(housing_offer.reload.ordered_picture_ids).to eq([picture1.id.to_s, picture2.id.to_s])
      end

      it 'update an existing offer without udpating lessor_status, offer_status or reserving_business_name' do 
        params = {
          data: {
            attributes: {
              external_ref: "111",
              lessor_siren: "555555555",
              lessor_code: "000",
              typology: 'T2',
              lessor_status: 'lol', 
              offer_status: 'bouh', 
              reserving_business_name: 'qdsqd', 
            }
          }
        }
        authenticated_header(request, user)
        post 'create', params
        attributes = params[:data][:attributes]
        expect(response).to have_http_status(200)
        expect(json['data']['id']).to eq(housing_offer.id.to_s)
        expect(json['data']['attributes']["typology"]).to eq(attributes[:typology])
        expect(json['data']['attributes']["lessor_status"]).not_to eq(attributes[:lessor_status])
        expect(json['data']['attributes']["offer_status"]).not_to eq(attributes[:offer_status])
        expect(json['data']['attributes']["reserving_business_name"]).not_to eq(attributes[:reserving_business_name])
      end 

      it 'create new one when create_new_anyway_status' do 
        housing_offer.update(offer_status: 'ARCHIVE')
        authenticated_header(request, user)
        post 'create',data
        expect(response).to have_http_status(201)
        expect(json['data']['id']).not_to eq(housing_offer.id.to_s)
        expect(json['data']['attributes']["typology"]).to eq("T2")
      end 
      it 'not duplicate if lessor_status= "REPRISE BAILLEUR A CONFIRMER"' do
        housing_offer.update(offer_status: 'ARCHIVE', lessor_status: "REPRISE BAILLEUR A CONFIRMER")
        authenticated_header(request, user)
        post 'create',data
        expect(response).to have_http_status(200)
        expect(json['data']['id']).to eq(housing_offer.id.to_s)
      end 
      
    end

    describe 'Create housing_offer autodelete' do
      let!(:lessor) {create(:lessor, user_id: user.id, title: 'Lessor')}

      let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor.id.to_s, auto_destroy_next_day: true,
                                    external_ref: "111", typology: "T1")}
      before :each do
        authenticated_header(request, user)
        post 'create',
              data: {
                attributes: {
                  lessor_id: lessor.id.to_s,
                  external_ref: "111",
                  auto_destroy_next_day: true,
                  typology: "T2"
                }
              }
        housing_offer.set(updated_at: Time.now, last_update_by_lessor: Time.now - 37.hours)
        drain_all_jobs

      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
        expect(housing_offer.reload.offer_status).to eq("RETIRE PAR LE BAILLEUR")
        expect(housing_offer.reload.lessor_status).to eq("REPRISE BAILLEUR A CONFIRMER")
        expect(json['data']['id']).to eq(housing_offer.id.to_s)
        expect(json['data']['attributes']["typology"]).to eq("T2")
      end
      
      describe 'POST /housing_offers as an admin' do
        before do
          authenticated_header(request, responsible)
          post :create, data: {
            attributes: {
              offer_status: 'requalification',
              lessor_id: lessor.id.to_s
            }
          }
        end
        it 'can not set reatribution status' do
          expect(response.status).to eq(201)
          expect(json['data']['attributes']['offer_status']).not_to eq('requalification')
          expect(json['data']['attributes']['requalification']).not_to eq(true)
        end
      end
    end

    describe 'with details' do 
      let!(:lessor) {create(:lessor, user_id: user.id, title: 'lessor for details', siren: "123456789")}
      let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_siren: lessor.siren, external_ref: "offer_with_details", typology: "T1", offer_status: 'PUBLIE')}
      before :each do
        authenticated_header(request, user)
      end

      it 'when post data nil' do 
        housing_offer.update(details: 'test')
        post 'create',
              data: {
                attributes: {
                lessor_siren: housing_offer.lessor_siren,
                  external_ref: housing_offer.external_ref,
                  details: nil
                }
              }
        expect(json['data']['id']).to eq(housing_offer.id.to_s)
        expect(housing_offer.reload.details).to eq('test')
      end 

      it 'when post data not blank' do 
        post 'create',
              data: {
                attributes: {
                lessor_siren: housing_offer.lessor_siren,
                  external_ref: housing_offer.external_ref,
                  details: 'lol'
                }
              }
        expect(json['data']['id']).to eq(housing_offer.id.to_s)
        expect(housing_offer.reload.details).to eq('lol')
      end 
    end 

    describe 'PUT /housing_offers/:id as an admin' do
      let!(:lessor_user) {create(:user, :lessor)}
      let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

      before do
        authenticated_header(request, responsible)
        put :update, id: housing_offer.id, data: {
          attributes: {
            offer_status: 'requalification'
          }
        }
      end

      it 'can not set reatribution status' do
        expect(response.status).to eq(200)
        expect(housing_offer.reload.offer_status).not_to eq('requalification')
        expect(housing_offer.reload.requalification).to be(false)
      end
    end

    describe 'PUT /housing_offers/:id republication dépublié as an admin' do
      let!(:lessor_user) {create(:user, :lessor)}
      let!(:housing_offer) {create(:housing_offer,  user_id: user.id, lessor_id: lessor_user.lessor.id.to_s,  lessor_status: "EN DESIGNATION", offer_status: "EN COURS DE DESIGNATION"  , external_ref: "111", typology: "T1", publication_end_date: DateTime.current - 1.days)}

      before do
        housing_offer.set(publication_end_date: Time.zone.now, offer_status: 'COMMERCIALISATION NON PUBLIEE',published: false, updated_at: DateTime.current)
        authenticated_header(request, responsible)
        put :update, id: housing_offer.id, "data":
        {"attributes":
          {"offer_status":"LOGEMENT PUBLIE",
            "lessor_status":"PUBLIE",
            "date_publication_start":
            "Thu Jun 17 00:00:00 CEST 2021",
            "publication_end_date": (DateTime.current + 10.days)}}
      end

      it 'is republished' do
        expect(response.status).to eq(200)
        expect(housing_offer.reload.offer_status).to eq('LOGEMENT PUBLIE')
      end
    end
    describe "pictures" do
      let!(:lessor_user) {create(:user, :lessor)}
      let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

      before do 
        authenticated_header(request, responsible)
      end 

      it 'pictures numbers when order picture and picture url' do
        picture = Hermes::Picture.create(housing_offer_id: housing_offer.id, thumb240_id: 'test')
        housing_offer.set(
            ordered_picture_ids: [
              picture.id.to_s
            ],
            picture_urls: [{
              thumb240: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
              thumb1000: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
              full_size: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg"
            },
            {
              thumb240: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
              thumb1000: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
              full_size: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg"
            }
          ]
        )
        get :show, id: housing_offer.id
        expect(json['data']['attributes']['pictures_nb']).to eq(3)
      end

      it 'shows google for default picture' do 
        get :show, id: housing_offer.id
        expect(json['data']['attributes']['main_picture']['thumb240'].match(/googleapis/)).to be_truthy
        expect(json['data']['attributes']['pictures'][0]['thumb240'].match(/googleapis/)).to be_truthy
      end 

      it 'shows picture' do 
        picture = Hermes::Picture.create(thumb240_id: 'test')
        housing_offer.update(ordered_picture_ids: [picture.id.to_s])        
        get :show, id: housing_offer.id
        expect(json['data']['attributes']['main_picture']['thumb240']).to eq('/no_preview_image.png?id=test')
        expect(json['data']['attributes']['pictures'][0]['thumb240']).to eq('/no_preview_image.png?id=test')
      end 
      
      it 'shows pictures_urls in json' do
        put :update, id: housing_offer.id, data: {
          attributes: {
            picture_urls: [{
              thumb240: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
              thumb1000: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
              full_size: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg"
            },
            {
              thumb240: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
              thumb1000: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
              full_size: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg"
            }
          ]
          }
        }
        expect(response.status).to eq(200)
        expect(json['data']['attributes']['main_picture']['thumb240']).to eq("https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg")
        expect(json['data']['attributes']['pictures'][0]['thumb240']).to eq("https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg")
      end

      it 'clear cache when adding picture' do 
        get :show, id: housing_offer.id
        picture = Hermes::Picture.create(thumb240_id: 'test', housing_offer_id: housing_offer.id)
        get :show, id: housing_offer.id
        expect(json['data']['attributes']['main_picture']['thumb240']).to eq('/no_preview_image.png?id=test')
        expect(json['data']['attributes']['pictures'][0]['thumb240']).to eq('/no_preview_image.png?id=test')
      end 

      describe 'PUT with picture_urls' do
        let!(:lessor_user) {create(:user, :lessor)}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

        before do
          authenticated_header(request, responsible)
          put :update, id: housing_offer.id, data: {
            attributes: {
              picture_urls: [{
                full_size: "https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg",
              }
            ]
            }
          }
        end

        it 'shows pictures in json' do
          expect(response.status).to eq(200)
          expect(json['data']['attributes']['main_picture']['thumb240']).to eq("https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg")
          expect(json['data']['attributes']['pictures'][0]['thumb240']).to eq("https://www.tuningblog.eu/wp-content/uploads/2020/05/Toyota-Motor-Stance-Tuning-Lada-Niva-1600-4x4-Camber-Header.jpg")
        end
      end
      describe 'PUT with wrong picture urlpicture_urls' do
        let!(:lessor_user) {create(:user, :lessor)}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

        before do
          authenticated_header(request, responsible)
          put :update, id: housing_offer.id, data: {
            attributes: {
              picture_urls: [{
                foo: "bar",
              }
            ]
            }
          }
        end

        it 'return errors' do
          expect(response.status).to eq(422)
        end
      end
      describe 'PUT with no picture_urls' do
        let!(:lessor_user) {create(:user, :lessor)}
        let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

        before do
          authenticated_header(request, responsible)
          put :update, id: housing_offer.id, data: {
            attributes: {
              picture_urls: nil
            }
          }
        end

        it 'dont return error return errors' do
          expect(response.status).to eq(200)
        end
      end
    end

    describe 'PUT /housing_offers/:id as superadministrateur' do
      let!(:lessor_user) {create(:user, :lessor)}
      let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_id: lessor_user.lessor.id.to_s, auto_destroy_next_day: true, external_ref: "111", typology: "T1")}

      before do
        authenticated_header(request, user)
        put :update, id: housing_offer.id, data: {
          attributes: {
            offer_status: 'requalification'
          }
        }
      end

      it 'can not set reatribution status' do
        expect(response.status).to eq(200)
        expect(housing_offer.reload.offer_status).not_to eq('requalification')
        expect(housing_offer.reload.requalification).to be(false)
      end
    end

    describe 'post request with construction_year is valid' do
      before :each do
        authenticated_header(request, responsible)
        post 'create',
              data: {
                attributes: {construction_year: '2001'}
              }
      end
      it 'returns status code 201 and the value appears in json' do
        expect(response).to have_http_status(201)
        expect(Date.parse(json['data']['attributes']['construction_year'])).to eq(Date.new(2001, 0o1, 0o1))
      end
    end
  end

  describe 'GET /housing_offers' do
    let!(:housing_offer1) {create(:housing_offer, terraces_or_balconies: 50, description: 'marseille', coordinates: [5.37569223, 43.29360776])}
    let!(:housing_offer3) {create(:housing_offer, terraces_or_balconies: 150, description: 'aix', coordinates: [5.44579067, 43.52887134])}
    let!(:housing_offer2) {create(:housing_offer, terraces_or_balconies: 150, description: 'lyon', coordinates: [4.81934988, 45.78210582])}
    [
      {user: 'responsible', total_objects: 3},
      {user: 'superadministrateur', total_objects: 3},
      {user: 'user', total_objects: 0},
    ].each do |a|
      describe "as #{a[:user]}" do
        before :each do
          authenticated_header(request, eval(a[:user]))
        end
        it 'reponse headers pagination' do 
          get 'index', per_page: 1
          page = a[:total_objects] > 0 ? a[:total_objects] : 1
          expect(response.headers['X-Total-Count']).to eq(a[:total_objects])
          expect(response.headers['X-Total-Pages']).to eq(page)
          expect(response.headers['Link']).to include("<http://test.host/api/dmo/housing_offers?per_page=1&page=1>; rel=\"first\"")
          expect(response.headers['Link']).to include("<http://test.host/api/dmo/housing_offers?per_page=1&page=#{page}>; rel=\"last\"")
          expect(response.headers['Link']).to include("<http://test.host/api/dmo/housing_offers?per_page=1&page=2>; rel=\"next\"") if a[:total_objects] > 0
        end 
        it 'returns a valid json' do
          get 'index'
          expect(response).to have_http_status(200)
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(a[:total_objects])
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(a[:total_objects])
        end
      end
    end

    # describe 'GET /housing_offers with filters eligibility' do
    #   let!(:user3) {create(:user, auth_roles: ['demandeur'])}
    #   let!(:applicant) {create(:applicant, user_id: user3.id.to_s )} #,\"floor_category_6\":{\"$lte\":68817.0}
    #   let!(:housing_request2) {create(:housing_request, main_applicant_id: applicant.id.to_s, is_active: true)}
    #   let!(:housing_offer1) {create(:housing_offer, typology: "T6",offer_status: 'PUBLIE')}
    #   let!(:housing_offer3) {create(:housing_offer, typology: "T2",offer_status: 'PUBLIE')}
    #   let!(:housing_offer2) {create(:housing_offer, typology: "T4",offer_status: 'PUBLIE')}

    #   before do 
    #     authenticated_header(request, user3)
    #     user3.set(main_applicant_id: applicant.id.to_s)
    #     housing_request2.set(eligibility_query_base: {typology:{"$in": ["T4","T5","T6","T7","T6P"]}}.to_json , eligibility_query_department: {}.to_json ,updated_at: DateTime.current)
    #   end 
      
    #   it "as #user with no params" do
    #     get 'index'
    #     expect(response).to have_http_status(200)
    #     expect(json).not_to be_empty
    #     expect(json['data'].length).to eq(2)
    #     expect(json['meta']['pagination']).to be_present
    #     expect(json['meta']['pagination']['total_objects']).to eq(2)
    #   end
    #   it "as #user with query params 0"  do
    #     get 'index', {'typology' => {'$in' => ['T2']}}
    #     expect(response).to have_http_status(200)
    #     expect(json).not_to be_empty
    #     expect(json['data'].length).to eq(0)
    #     expect(json['meta']['pagination']).to be_present
    #     expect(json['meta']['pagination']['total_objects']).to eq(0)
    #   end
    #   it "as #user with query params" do
    #     get 'index', {'typology' => {'$in' => ['T2', 'T4']}}
    #     expect(response).to have_http_status(200)
    #     expect(json).not_to be_empty
    #     expect(json['data'].length).to eq(1)
    #     expect(json['meta']['pagination']).to be_present
    #     expect(json['meta']['pagination']['total_objects']).to eq(1)
    #   end
    # end

    describe 'Find by location' do
      let!(:admin) { create(:administrator) }
      before :each do
        Hermes::HousingOffer.remove_indexes
        Hermes::HousingOffer.create_indexes
        authenticated_header(request, admin)
      end

      it 'should return houssing_offers on the selected area' do
        # Coordinates is for Montelimart
        get :index, longitude: 44.55127, latitude: 4.75386, distance: 130
        expect(json['data'].count).to eq(1)
        # Lyon is less than 130 kilometers away
        expect(json['data'].first['id']).to eq(housing_offer3.id.to_s)
      end
    end
  end

  describe 'GET /housing_offers' do 

    describe 'eligible_count option' do
      let!(:offer) { create(:published_offer, insee_code: '34210', typology: 'T1', threshold_type: 'PLUS', threshold_subtype: 'Paris et communes limitrophes') }
      let!(:hr) { create(:complete_request, seeked_district_insee_codes: ['34210', '34100'])}

      it 'not used by default' do 
        authenticated_header(request, superadministrateur)
        get 'show', {id: offer.id}
        expect(response).to have_http_status(200)
        expect(json["data"]["attributes"]["eligible_count"]).to eq(nil)
      end 

      it 'works' do 
        authenticated_header(request, superadministrateur)
        get 'show', {id: offer.id, options: [:eligible_count]}
        expect(response).to have_http_status(200)
        expect(json["data"]["attributes"]["eligible_count"]["department_count"]).to eq(1)
        expect(json["data"]["attributes"]["eligible_count"]["seeked_count"]).to eq(1)
        expect(json["data"]["attributes"]["eligible_count"]["bordering_count"]).to eq(0)
        expect(json["data"]["attributes"]["eligible_count"]["seeked_bordering_count"]).to eq(1)
        expect(json["data"]["attributes"]["eligible_count"]["other_from_department_count"]).to eq(0)
        expect(json["data"]["attributes"]["eligible_count"]["total_count"]).to eq(1)
      end

      it 'with estimate' do 
        authenticated_header(request, superadministrateur)
        offer.set(
          async_eligible_requests_nb: 12589,
          async_eligible_department_nb: 3589,
          async_eligible_seeked_nb: 4589,
          async_eligible_bordering_nb: 5589,
          async_eligible_seeked_bordering_nb: 6589,
          async_eligible_other_from_department_nb: 7589,
        )
        get 'show', {id: offer.id, options: [:eligible_count]}
        expect(response).to have_http_status(200)
        expect(json["data"]["attributes"]["eligible_count"]["department_count"]).to eq(3000)
        expect(json["data"]["attributes"]["eligible_count"]["seeked_count"]).to eq(4000)
        expect(json["data"]["attributes"]["eligible_count"]["bordering_count"]).to eq(5000)
        expect(json["data"]["attributes"]["eligible_count"]["seeked_bordering_count"]).to eq(6000)
        expect(json["data"]["attributes"]["eligible_count"]["other_from_department_count"]).to eq(7000)
        expect(json["data"]["attributes"]["eligible_count"]["total_count"]).to eq(12000)
      end 
    end
  end 

  describe 'GET /housing_offers/id/all_applications' do
    let!(:housing_offer1) {create(:housing_offer, terraces_or_balconies: 50)}
    let!(:housing_offer2) {create(:housing_offer, terraces_or_balconies: 50)}
    let!(:user) {create(:user)}
    (1..6).each do |i|
      let!("applicant#{i}".to_sym) {create(:applicant, birth_date: Date.today - 50.years)}
      let!("housing_request#{i}".to_sym) {create(:housing_request, is_active: true, motivations: ['acceuillant'], documents_required: 50, main_applicant_id: send("applicant#{i}").id.to_s)}
    end
    let!(:housing_application1) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                        housing_request_id: housing_request1.id.to_s)}
    let!(:housing_application2) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                        housing_request_id: housing_request2.id.to_s)}
    let!(:housing_application3) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                        housing_request_id: housing_request3.id.to_s)}
    let!(:housing_application4) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                        housing_request_id: housing_request4.id.to_s, is_active: false)}
    let!(:housing_application5) {create(:housing_application, housing_offer_id: housing_offer2.id.to_s,
                                        housing_request_id: housing_request5.id.to_s)}
    let!(:housing_application6) {create(:housing_application, housing_offer_id: housing_offer1.id.to_s,
                                        housing_request_id: housing_request6.id.to_s, rank: 1)}
  
    it 'return valid json without sort' do
      authenticated_header(request, responsible)
      get 'all_applications', {id: housing_offer1.id}
      expect(response).to have_http_status(200)
      expect(json['data'].length).to eq(4)
    end

    it 'filter on rank' do
      authenticated_header(request, responsible)
      get 'all_applications', {id: housing_offer1.id, rank: 1}
      expect(response).to have_http_status(200)
      expect(json['data'].length).to eq(1)
    end
  end

  Hermes::Api::Dmo::HousingOffersController.acceptable_params.sort.each do |acceptable_param, options|
    type = options[:type]
    next unless type
    next if ['ids'].include?(acceptable_param.to_s)

    describe "GET /housing_offers with sort on field #{acceptable_param}" do
      let!(:housing_offer3) {create(:housing_offer)}
      let!(:housing_offer1) {create(:housing_offer)}
      let!(:housing_offer2) {create(:housing_offer)}

      before :each do
        (1..3).each do |i|
          record = eval("housing_offer#{i}")
          record.unset(acceptable_param.to_sym)
          record.set(acceptable_param.to_sym => get_typed_value(type, (i+1) % 3))
        end 

        authenticated_header(request, responsible)
        get 'index', sort: {acceptable_param => 1}
      end

      it 'returns sorted results' do
        attribute = if type == 'DateTime'
          json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param.to_s])}
        elsif type == 'Date'
          json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param.to_s])}
        else
          json['data'].map {|unit| unit['attributes'][acceptable_param.to_s]}
        end
        expect(response.status).to eq(200)
        expect(attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
      end
    end
  end

  describe 'GET /housing_offers with operators' do
    let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}
    let!(:housing_offer3) {create(:housing_offer, description: 'Super', terraces_or_balconies: 150)}
    let!(:housing_offer2) {create(:housing_offer, description: 'bel appart', terraces_or_balconies: 150)}
    describe 'with $lt' do
      before :each do
        authenticated_header(request, responsible)
        get 'index', terraces_or_balconies: {'$lt': 100}
      end
      it 'returns results' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end
    describe 'with $in' do
      before :each do
        authenticated_header(request, responsible)
        get 'index', description: {'$in': %w[acceuillant flamand]}
      end
      it 'returns results' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end
    describe 'with $ne' do
      before :each do
        authenticated_header(request, responsible)
        get 'index', terraces_or_balconies: {'$ne': 50}
      end
      it 'returns sorted results' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq(2)
      end
    end
    describe 'with $regex' do
      before :each do
        authenticated_header(request, responsible)
        get 'index', description: {'$regex': /^S/}
      end
      it 'returns sorted results' do
        expect(response.status).to eq(200)
        expect(json['meta']['pagination']['total_objects']).to eq(1)
      end
    end
  end

  describe 'GET /housing_offers search by postal codes' do
    let!(:offers_1) {create_list :housing_offer, 2, postal_code: '93240'}
    let!(:offers_rand) {create_list :housing_offer, 4, postal_code: '2'.ljust(5, rand(9999).to_s)}
    let!(:offers_2) {create_list :housing_offer, 2, postal_code: '75100'}

    before do
      authenticated_header(request, responsible)
      get 'index', postal_codes: ['75100', '93240']
    end

    it 'succeeds' do
      expect(response.status).to eq(200)
      expect(json['meta']['pagination']['total_objects']).to eq 8
    end

    it 'orders results' do
      expect(json['data'][0]['attributes']['postal_code']).to eq '75100'
      expect(json['data'][2]['attributes']['postal_code']).to eq '93240'
    end
  end

  describe 'GET /housing_offers with eligible_requests_nb & possible_eligible_requests_nb' do
    let!(:offer) {create(:housing_offer, :with_eligible_requests, requests_count: 3)}

    before do
      authenticated_header(request, responsible)
      offer.eligibles_requests.first.set(request_status: 'INCOMPLETE')
    end

    it 'add attributes' do
      get 'index', options: ['eligible_requests_nb']
      expect(response.status).to eq(200)
      expect(json['data'].length).to eq(1)
      expect(json['data'][0]['attributes']['eligible_requests_nb']).to eq(2)
      expect(json['data'][0]['attributes']['possible_eligible_requests_nb']).to eq(nil)
    end

    it 'with options possible_eligible_requests_nb' do
      get 'index', options: ['possible_eligible_requests_nb']
      expect(response.status).to eq(200)
      expect(json['data'].length).to eq(1)
      expect(json['data'][0]['attributes']['eligible_requests_nb']).to eq(nil)
      expect(json['data'][0]['attributes']['possible_eligible_requests_nb']).to eq(3)
    end

    it 'with options eligible_requests_nb & possible_eligible_requests_nb' do
      get 'index', options: ['eligible_requests_nb' ,'possible_eligible_requests_nb']
      expect(response.status).to eq(200)
      expect(json['data'].length).to eq(1)
      expect(json['data'][0]['attributes']['eligible_requests_nb']).to eq(2)
      expect(json['data'][0]['attributes']['possible_eligible_requests_nb']).to eq(3)
    end

    it 'missing attributes' do
      get 'index'
      expect(response.status).to eq(200)
      expect(json['data'].length).to eq(1)
      expect(json['data'][0]['attributes']['eligible_requests_nb'].present?).to eq(false)
      expect(json['data'][0]['attributes']['possible_eligible_requests_nb'].present?).to eq(false)
    end

    it 'with show' do 
      get 'show', {id: offer.id, options: ['eligible_requests_nb']}
      expect(response.status).to eq(200)
      expect(json['data']['attributes']['eligible_requests_nb']).to eq(2)
      expect(json['data']['attributes']['possible_eligible_requests_nb']).to eq(nil)
    end 

    it 'with estimate' do 
      offer.set(async_eligible_requests_nb: 32889)
      get 'show', {id: offer.id, options: ['eligible_requests_nb']}
      expect(response.status).to eq(200)
      expect(json['data']['attributes']['eligible_requests_nb']).to eq(32000)
    end 

    it 'with estimate and config' do 
      offer.set(async_eligible_requests_nb: 32889)
      HERMES_CONFIG['eligible_request_estimate_min'] = 20000
      HERMES_CONFIG['eligible_request_estimate_step'] = 10

      get 'show', {id: offer.id, options: ['eligible_requests_nb']}
      expect(response.status).to eq(200)
      expect(json['data']['attributes']['eligible_requests_nb']).to eq(32880)
      
      # reset config
      HERMES_CONFIG['eligible_request_estimate_min'] = nil
      HERMES_CONFIG['eligible_request_estimate_step'] = nil
    end 

  end

  describe 'POST /housing_offers/search' do
    let!(:housing_offer) {create(:housing_offer, typology: "T1")}
    before :each do
      authenticated_header(request, superadministrateur)
      post 'search', {typology: "T1"}
    end
    
    it 'returns status code 200' do
      expect(response).to have_http_status(200)
    end
    
    it 'returns a valid json' do
      expect(json['data'].length).to eq(1)
    end
  end

  describe 'GET /housing_offers/id/eligible_requests' do 
    let!(:entity1) {create(:entity, siren: '123456789', siret: '12345678901234')}
    let!(:entity2) {create(:entity, parent_id: entity1.id,  siret: '22345678901235') }
    let!(:entity3) {create(:entity, siret: '32345678901236') }
    let!(:rh) { create(:reservation_holder, siren: [entity1.siren])}
    let!(:offer) {create(:housing_offer, :with_eligible_requests, requests_count: 5)}

    before do :each 
      authenticated_header(request, superadministrateur)
    end 
    
    it 'works' do 
      get 'eligible_requests', id: offer.id
      expect(json['data'].length).to eq(5)
    end 

    it 'works with filter' do 
      offer.eligibles_requests.order_by(created_at: :desc).first.set(housing_channel_keys: ['dc', 'dalo']) 
      offer.eligibles_requests.order_by(created_at: :desc).last.set(housing_channel_keys: ['test']) 

      authenticated_header(request, responsible)
      get 'eligible_requests', id: offer.id
      expect(json['data'].length).to eq(4)
      get 'eligible_requests', id: offer.id, housing_channel_keys: 'dalo'
      expect(json['data'].length).to eq(1)
      get 'eligible_requests', id: offer.id, housing_channel_keys: 'test'
      expect(json['data'].length).to eq(0)
    end 
    
    it 'with reserving_siren' do 
      offer.eligibles_requests.each_with_index do |hr, i|
        hr.set(request_status: 'COMPLETE', entity_id:  eval("entity#{i+1}.id"))if i < 3
      end 
      get 'eligible_requests', id: offer.id, reserving_siren: rh.siren.last
      expect(json['data'].length).to eq(2)
    end 

    it 'with entity_id' do 
      offer.eligibles_requests.first.set(entity_id: entity1.id)
      get 'eligible_requests', id: offer.id, entity_id: entity1.id.to_s
      expect(json['data'].length).to eq(1)
    end
    
    it 'works with type and count' do 
      offer.eligibles_requests.each_with_index do |hr, i|
        hr.set(seeked_district_insee_codes: [offer.insee_code])  if i < 2
        hr.set(seeked_district_insee_codes: [offer.insee_code + '1'])  if i == 3
      end 

      get 'eligible_requests', id: offer.id, eligibility_type: 'seeked', options: ['other_from_department_count', 'bordering_count']
      expect(json['data'].length).to eq(4)
      expect(json['data'].first['attributes']['is_active']).to eq(true)
      expect(json['meta']['other_from_department_count']).to eq(1)
      expect(json['meta']['bordering_count']).to eq(0)
    end 

  end 

  describe 'GET /housing_offers/:id/is_eligible_request' do
    let!(:offer) {create(:housing_offer, :with_eligible_requests, requests_count: 1)}
    let!(:eligible_request) { offer.eligibles_requests.first}
    let!(:other_request) { create(:active_request)}

    before :each do 
      authenticated_header(request, responsible)
      eligible_request.set(als_id: '000000001')
      other_request.set(als_id: '000000002')
    end 

    it 'not found instance' do 
      get 'is_eligible_request', id: 'not found', als_id: eligible_request.als_id.to_s
      expect(response).to have_http_status(404)
      expect(json['errors'][0]['title']).to include('instance_not_found')
    end 

    it 'missing offer_id' do 
      get 'is_eligible_request', id: offer.id.to_s, als_id: ''
      expect(response).to have_http_status(406)
      expect(json['errors'][0]['title']).to include('missing_als_id')
    end 

    it 'not found offer' do 
      get 'is_eligible_request', id: offer.id.to_s, als_id: 'not found'
      expect(response).to have_http_status(404)
      expect(json['errors'][0]['title']).to include('request_not_found')
    end 

    it 'when eligible' do 
      get 'is_eligible_request', id: offer.id.to_s, als_id: eligible_request.als_id.to_s
      expect(response).to have_http_status(200)
      expect(json['is_eligible']).to eq(true)
    end 

    it 'when not eligible' do 
      get 'is_eligible_request', id: offer.id.to_s, als_id: other_request.als_id.to_s
      expect(response).to have_http_status(200)
      expect(json['is_eligible']).to eq(false)
    end 
  end 

  describe 'GET /housing_offers/reference_exist' do
    let!(:housing_offer1) {create(:housing_offer, external_ref: 'test', offer_status: Hermes::HousingOffer.end_of_life_statuses.last)}
    let!(:housing_offer2) {create(:housing_offer, external_ref: 'test', offer_status: 'AUTRE')}
    let!(:housing_offer1) {create(:housing_offer, external_ref: 'bouh',)}

    it 'succeeds' do
      authenticated_header(request, responsible)
      get 'reference_exist', { external_ref: 'test' }
      expect(response.status).to eq(200)
      expect(json['meta']['pagination']['total_objects']).to eq 1
    end
  end

  describe 'GET /housing_offers/:id' do
    describe 'verif it work' do
      let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}
      before :each do
        authenticated_header(request, responsible)
        get 'show', id: housing_offer1.id
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end

    describe 'verif it work not work with a non authorized user' do
      let!(:housing_offer1) {create(:housing_offer, description: 'acceuillant', terraces_or_balconies: 50)}

      before :each do
        authenticated_header(request, user)
        get 'show', id: housing_offer1.id
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(404)
      end
    end

    context 'with housing applications' do
      let!(:threshold1) do
        create(
          :threshold,
          type: 'PLA LM',
          thresholds_map: {
            'Paris et communes limitrophes': {
              'thresholds_list': [
                13_050,
                21_272,
                27_883,
                30_521,
                36_307,
                40_859
              ],
              'another_person': 4552
            },
            'IDF hors Paris et Communes limitrophes': {
              'thresholds_list': [
                13_050,
                21_272,
                25_569,
                28_075,
                33_238,
                37_401
              ],
              'another_person': 4166
            },
            'Autres régions': {
              'thresholds_list': [
                11_342,
                16_525,
                19_872,
                22_111,
                25_870,
                29_155
              ],
              'another_person': 3252
            }
          }
        )
      end

      let!(:applicant1) do
        create(
          :applicant,
          amount_year_minus_two: 10_346.0,
          monthly_incomes: {
            'SAL': 864.32
          }
        )
      end
      let!(:request1) do
        create(
          :housing_request,
          main_applicant: applicant1,
          is_active: true,
          seeked_departments: %w[12]
        )
      end

      let!(:applicant2) do
        create(
          :applicant,
          amount_year_minus_two: 10_346.0,
          monthly_incomes: {
            'SAL': 864.32
          }
        )
      end
      let!(:request2) do
        create(
          :housing_request,
          main_applicant: applicant2,
          is_active: true,
          seeked_departments: %w[12]
        )
      end

      let!(:applicant3) do
        create(
          :applicant,
          amount_year_minus_two: 10_346.0,
          monthly_incomes: {
            'SAL': 864.32
          }
        )
      end
      let!(:request3) do
        create(
          :housing_request,
          main_applicant: applicant3,
          is_active: true,
          seeked_departments: %w[12]
        )
      end

      let!(:offer1) do
        create(
          :housing_offer,
          department: '12',
          typology: 'T2',
          threshold_type: 'PLA LM',
          threshold_subtype: 'Autres régions',
          rent_with_charges: 324.12
        )
      end
      let!(:offer2) do
        create(
          :housing_offer,
          department: '12',
          typology: 'T2',
          threshold_type: 'PLA LM',
          threshold_subtype: 'Autres régions',
          rent_with_charges: 324.12
        )
      end

      let!(:application1_1) do
        create(
          :housing_application,
          housing_request: request1,
          housing_offer: offer1,
          status: 'TRANSMIS BAILLEUR'
        )
      end
      let!(:application1_2) do
        create(
          :housing_application,
          housing_request: request1,
          housing_offer: offer2,
          status: 'TRANSMIS BAILLEUR'
        )
      end

      let!(:application2_1) do
        create(
          :housing_application,
          housing_request: request2,
          housing_offer: offer1,
          status: 'TRANSMIS BAILLEUR'
        )
      end
      let!(:application2_2) do
        create(
          :housing_application,
          housing_request: request2,
          housing_offer: offer2,
          status: 'TRANSMIS BAILLEUR'
        )
      end

      let!(:application3_1) do
        create(
          :housing_application,
          housing_request: request3,
          housing_offer: offer1,
          status: 'TRANSMIS BAILLEUR'
        )
      end

      let!(:application3_2) do
        create(
          :housing_application,
          housing_request: request3,
          housing_offer: offer1,
          status: 'TRANSMIS BAILLEUR',
          is_active: false,
        )
      end

      describe 'GET /with_validated_applications without from' do
        before :each do
          authenticated_header(request, responsible)
          get 'with_validated_applications'
        end

        it 'returns error' do
          expect(response).to_not have_http_status(200)
        end
      end

      describe 'GET /with_validated_applications' do
        before :each do
          authenticated_header(request, responsible)
          get 'with_validated_applications', from: (Time.now - 1.minute).to_s
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to be 2
        end
      end

      describe 'GET /with_validated_applications without pagination' do
        before :each do
          authenticated_header(request, responsible)
          get 'with_validated_applications', from: (Time.now - 1.minute).to_s, pagination: 'false', per_page: '1'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to be 2
        end
      end

      describe 'GET /with_validated_applications/offer1.id' do
        before :each do
          authenticated_header(request, responsible)
          get 'with_validated_applications_for', id: offer1.id.to_s, from: (Time.now - 1.hours).to_s
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to be 4
        end
      end

      describe 'GET /with_validated_applications/offer1.id without pagination' do
        before :each do
          authenticated_header(request, responsible)
          get 'with_validated_applications_for', id: offer1.id.to_s, from: (Time.now - 1.hours).to_s, pagination: 'false', per_page: '1'
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to be 4
        end
      end

      describe 'GET /with_validated_applications/offer2.id' do
        before :each do
          authenticated_header(request, responsible)
          get 'with_validated_applications_for', id: offer2.id.to_s, from: (Time.now - 1.hours).to_s
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to be 2
        end
      end
    end
  end

  describe 'GET /housing_offers/:id/entity_statuses' do
    let!(:housing_offer1) {create(:housing_offer)}
    let!(:user1) {create(:user)}
    let!(:user2) {create(:user)}
    let!(:entity1) {create(:entity, siret: '00000000000001', readers:[user1.id.to_s])}
    let!(:entity2) {create(:entity, siret: '00000000000002')}

    let!(:status1) {create(:status,
      object_id: housing_offer1.id.to_s,
      status_label: 'entity_status',
      performer_id: entity1.id.to_s,
      title: 'Status1'
    )}

    let!(:status2) {create(:status,
      object_id: housing_offer1.id.to_s,
      status_label: 'entity_status',
      performer_id: entity1.id.to_s,
      title: 'Status2'
    )}

    let!(:status3) {create(:status,
      object_id: housing_offer1.id.to_s,
      status_label: 'entity_status',
      performer_id: entity2.id.to_s,
      title: 'Status3'
    )}

    describe 'missing params entity_id' do
      before :each do
        authenticated_header(request, user1)
        get 'entity_statuses', {
          id: housing_offer1.id
        }
      end

      it 'returns status code 406' do
        expect(response).to have_http_status(406)
      end
    end

    describe 'not found' do
      before :each do
        authenticated_header(request, user1)
        get 'entity_statuses', {
          id: 'notfound'
        }
      end

      it 'returns status code 404' do
        expect(response).to have_http_status(404)
      end
    end

    describe 'unauthorized' do
      before :each do
        authenticated_header(request, user2)
        get 'entity_statuses', {
          id: housing_offer1.id,
          entity_id: entity1.id
        }
      end

      it 'returns status code 401' do
        expect(response).to have_http_status(401)
      end
    end

    describe 'success' do
      before :each do
        authenticated_header(request, user1)
        get 'entity_statuses', {
          id: housing_offer1.id,
          entity_id: entity1.id
        }
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end

      it 'valid data' do
        expect(json['data'].length).to eq(2)
      end
    end
  end

  describe 'PATCH /housing_offers/:id' do
    housing_offer_params = {}
    describe 'sanitize_string_as_integer' do 
      let!(:housing_offer) {create(:housing_offer)}

      before :each do
        authenticated_header(request, responsible)
      end

      [1, 'O'].each do |type|
        it "works for #{type.class} #{type}" do 
          patch 'update', {id: housing_offer.id.to_s, data: {attributes: {elevators: type}}}
          expect(housing_offer.reload.elevators).to eq 1
        end 
      end 

      [nil, 0, 'N'].each do |type|
        it "works for #{type.class} #{type}" do 
          patch 'update', {id: housing_offer.id.to_s, data: {attributes: {elevators: type}}}
          expect(housing_offer.reload.elevators).to eq 0
        end 
      end 

    end 
  end

  describe 'PATCH /housing_offers/:id flux 11 & 13' do
    describe 'Update housing_offer offer_status and lessor_status' do
      let!(:applicant) {create(:applicant)}
      let!(:lessor) {create(:lessor, user_id: user.id, title: 'Lessor', siren: "555555555")}
      let!(:housing_offer) {create(:housing_offer, user_id: user.id, lessor_siren: "555555555", auto_destroy_next_day: true,
                                    external_ref: "111", typology: "T1")}
      let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id)}
      let!(:application) {create(:housing_application, housing_offer_id: housing_offer.id, housing_request_id: housing_request.id)}

      context 'flux 11 RENDU AU BAILLEUR' do 
        before :each do
          authenticated_header(request, user)
          patch 'update',
            id: housing_offer.id.to_s,
            data: {
              attributes: {
                offer_status: 'RENDU AU BAILLEUR',
                lessor_status: 'RENDU AU BAILLEUR CONFIRME'
              }
            }
          drain_all_jobs
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
          expect(application.reload.status).to eq('ANNULEE BAILLEUR')
          expect(application.reload.is_active).to be(false)
        end
      end
    end
  end


  describe 'patch request with change in ordered_picture_ids is valid' do
    let!(:housing_offer1) {create(:housing_offer, user_id: responsible.id)}
    let!(:picture0) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
    let!(:picture1) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
    let!(:picture2) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
    let!(:picture3) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}
    let!(:picture4) {create(:picture, housing_offer_id: housing_offer1.id.to_s, user_id: responsible.id)}

    before :each do
      authenticated_header(request, responsible)
      patch 'update',
            id: housing_offer1.id.to_s,
            data: {
              attributes: {
                ordered_picture_ids: [picture4.id.to_s, picture3.id.to_s, picture2.id.to_s, picture1.id.to_s, picture0.id.to_s]
              }
            }
    end
    it 'returns status code 200 and the value appears in json' do
      expect(response).to have_http_status(200)
      expect(json['data']['attributes']['ordered_picture_ids']).to eq([picture4.id.to_s, picture3.id.to_s, picture2.id.to_s, picture1.id.to_s, picture0.id.to_s])
    end
  end

  describe 'post selected application' do
    let!(:housing_offer1) {create(:housing_offer)}
    let!(:housing_offer2) {create(:housing_offer)}

    let!(:user7) {create(:user)}
    let!(:applicant7) {create(:applicant, user: user7)}
    let!(:housing_request7) {create(:housing_request, is_active: true, user: user7, main_user_id: user7.id, main_applicant_id: applicant7.id.to_s, unique_numbers: ['1'])}
    let!(:housing_applications7) do
      create(:housing_application,
              housing_offer_id: housing_offer1.id.to_s,
              housing_request_id: housing_request7.id.to_s,
              status: 'PLO')
    end
    let!(:user8) {create(:user)}
    let!(:applicant8) {create(:applicant, user: user8)}
    let!(:housing_request8) {create(:housing_request, is_active: true, user: user8, main_user_id: user8.id,  main_applicant_id: applicant8.id.to_s, unique_numbers: ['2'])}
    let!(:housing_applications8) do
      create(:housing_application,
              housing_offer_id: housing_offer1.id.to_s,
              housing_request_id: housing_request8.id.to_s,
              status: 'PLO')
    end

    before :each do
      authenticated_header(request, responsible)
    end
    it 'return valid json' do
      post 'post_selected_applications', id: housing_offer1.id, data: {attributes: {housing_application_ids: [housing_applications7.id, housing_applications8.id]}}
      expect(response).to have_http_status(200)
      expect(json).not_to be_empty
      data = json['data']
      expect(data.length).to eq(2)
      expect(housing_applications7.reload.status).to eq('TRANSMIS BAILLEUR')
      expect(housing_offer1.reload.offer_status).to eq("EN COURS D'ATTRIBUTION")
      expect(housing_offer1.reload.lessor_status).to eq('EN ATTRIBUTION')
    end

    it 'send error when missing params' do
      post 'post_selected_applications', id: housing_offer1.id, data: {attributes: {lol: [housing_applications7.id, housing_applications8.id]}}
      expect(response).to have_http_status(422)
    end

    it 'send errors when not active, found or on housing_offer' do 
      housing_applications7.update(status: 'ANNULEE')
      housing_applications8.update(housing_offer: housing_offer2)
      post 'post_selected_applications', {
        id: housing_offer1.id, 
        data: {
          attributes: {
            housing_application_ids: [
              housing_applications7.id, 
              housing_applications7.id, 
              housing_applications8.id,
              housing_applications8.id,
              'not_an_id'
            ]
          }
        }
      }
      expect(response).to have_http_status(422)
      expect(json[0]).to eq("Candidature (id : #{housing_applications7.id} / als_id : #{housing_applications7.als_id}) inactive, statut : ANNULEE, nud : 1")
      expect(json[1]).to eq("Candidature (id : #{housing_applications8.id} / als_id : #{housing_applications8.als_id}) non associée à cette offre (#{housing_offer1.id}), statut : PLO, nud : 2")
      expect(json[2]).to eq("Candidature (not_an_id) introuvable")
    end 

    it 'update other application' do 
      post 'post_selected_applications', id: housing_offer1.id, data: {attributes: {housing_application_ids: [housing_applications7.id,]}}
      expect(response).to have_http_status(200)
      expect(json['data'].length).to eq(1)
      expect(housing_applications8.reload.status).to eq('NON RETENUE')
      expect(housing_applications8.reload.is_active).to eq(false)
    end 

  end

  describe 'DELETE /housing_offers/:id' do
    let!(:housing_offer) {create(:housing_offer, user_id: responsible.id.to_s)}
    before :each do
      authenticated_header(request, responsible)

      delete 'destroy',
              id: housing_offer.id.to_s
    end
    it 'returns status code 404' do
      expect(response).to have_http_status(204)
    end
  end

  describe 'Housing channels management' do
    let!(:channel_1) { create :housing_channel, key: 'dalo' }
    let!(:channel_2) { create :housing_channel, key: 'priority_public' }
    let!(:channel_3) { create :housing_channel, key: 'droits_communs' }
    let!(:channel_4) { create :housing_channel, key: 'key1' }
    let!(:channel_5) { create :housing_channel, key: 'key2' }
    let!(:channel_6) { create :housing_channel, key: 'key3' }
    let!(:township_attachment) { create :housing_channel, key: 'township_attachment' }
    let!(:channel_setting) { create :housing_channel_setting, department: '34', housing_channel_keys: ['dalo', 'priority_public', 'key1', 'key2', 'key3', 'township_attachment'] }

    let(:roles) { ['superadministrateur', 'demandeur', 'entreprise', 'entreprise administrateur', 'collaborateur offre logement'] }
    let(:reservation) { create :reservation_holder }
    let(:lessor) { create :lessor }
    let(:offer) { create :published_offer, housing_channel_keys: ['dalo'], reservation_holder_id: reservation.id, lessor_id: lessor.id }

    before { authenticated_header(request, user) }

    shared_examples 'can add' do
      it 'requires key parameter' do
        post 'add_housing_channel', { id: offer.id.to_s }
        expect(response).to have_http_status 406
        expect(json['errors'][0]['message']).to eq 'param is missing or the value is empty: key or keys'
      end

      it 'returns not found channel error' do
        post 'add_housing_channel', { id: offer.id.to_s, key: 'unknown' }
        expect(response).to have_http_status 404
        expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
      end

      it 'returns already added error' do
        post 'add_housing_channel', {id: offer.id.to_s, key: 'dalo'}
        expect(response).to have_http_status 409
        expect(json['errors'][0]['message']).to eq "La filière logement 'dalo' a déjà été ajoutée."
      end

      it 'returns unavailable for department error' do
        post 'add_housing_channel', { id: offer.id.to_s, key: 'droits_communs' }
        expect(response).to have_http_status 422
        expect(json['errors'][0]['message']).to eq "La filière logement 'droits_communs' n'est pas disponible pour le département '34' recherché."
      end

      it 'succeeds with keys' do
        expect do
          post 'add_housing_channel', { id: offer.id.to_s, keys: ['priority_public'] }
        end.to change { offer.reload.housing_channel_keys }.from(['dalo']).to(['priority_public'])
        expect(response).to have_http_status 200
      end

      it 'succeeds with key' do
        expect do
          post 'add_housing_channel', { id: offer.id.to_s, key: 'priority_public' }
        end.to change { offer.reload.housing_channel_keys }.from(['dalo']).to(['dalo', 'priority_public'])
        expect(response).to have_http_status 200
      end

      it 'succeeds with multiple keys' do
        expect do
          post 'add_housing_channel', { id: offer.id.to_s, keys: ['key1', 'key2', 'key3'] }
        end.to change { offer.reload.housing_channel_keys }.from(['dalo']).to(['key1', 'key2', 'key3'])
        expect(response).to have_http_status 200
      end

      it 'succeeds with township attachement' do
        offer.update(housing_channel_keys: ['dc'])
        expect do
          post 'add_housing_channel', { id: offer.id.to_s, key: 'township_attachment' }
        end.to change { offer.reload.housing_channel_keys }.from(['dc']).to(['dc', 'township_attachment'])
        expect(response).to have_http_status 200
      end

      it 'can add double channel' do
        expect do
          post 'add_housing_channel', { id: offer.id.to_s, keys: ['priority_public',  'priority_public']}
        end.to change { offer.reload.housing_channel_keys }.from(['dalo']).to(['priority_public'])
        expect(response).to have_http_status 200
      end
    end

    shared_examples 'can remove' do
      it 'requires key parameter' do
        post 'remove_housing_channel', { id: offer.id.to_s }
        expect(response).to have_http_status 406
        expect(json['errors'][0]['message']).to eq 'param is missing or the value is empty: key or keys'
      end

      it 'returns not found channel error' do
        post 'remove_housing_channel', { id: offer.id.to_s, keys: ['unknown'] }
        expect(response).to have_http_status 404
        expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
      end

      it 'succeeds' do
        expect do
          post 'remove_housing_channel', { id: offer.id.to_s, keys: ['dalo'] }
        end.to change { offer.reload.housing_channel_keys }.from(['dalo']).to(['dc'])
        expect(response).to have_http_status 200
      end
    end

    shared_examples 'cannot add or remove' do
      it 'returns unauthorized on add' do
        post 'add_housing_channel', { id: offer.id.to_s, keys: ['droits_communs'] }
        expect(response).to have_http_status 401
      end
      it 'returns unauthorized on remove' do
        post 'remove_housing_channel', { id: offer.id.to_s, keys: ['dalo'] }
        expect(response).to have_http_status 401
      end
    end

    context 'as bailleur' do
      let(:user) { create :user, auth_roles: ['bailleur'] }
      it_behaves_like 'cannot add or remove'
      it_behaves_like 'can add' do
        before { lessor.update(user_id: user.id) }
      end
      it_behaves_like 'can remove' do
        before { lessor.update(user_id: user.id) }
      end
    end

    context 'as Administrator' do
      let(:user) { create :user, roles: ['Administrator'] }
      it_behaves_like 'can add'
      it_behaves_like 'can remove'
    end

    context 'as other role' do
      it_behaves_like 'cannot add or remove' do
        let(:user) { create(:user, auth_roles: roles.sample(rand(1..3))) }
      end
    end
  end

  context 'with basic data' do
    let!(:applicant) {create(:applicant, user: user)}
    let!(:housing_request) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant,
        user: user
      )
    end
    let!(:user5) {create(:user, auth_roles: ['collaborateur offre logement'])}
    let!(:lessor) {create(:lessor, siren: '987654321')}

    let!(:housing_offer1) do
      create(
        :housing_offer,
        user: responsible,
        lessor: lessor
      )
    end
    let!(:housing_offer2) do
      create(
        :housing_offer,
        user: responsible,
        lessor: lessor
      )
    end
    let!(:housing_offer3) do
      create(
        :housing_offer,
        user: responsible,
        lessor: lessor
      )
    end

    describe 'GET /housing_offers' do
      before :each do
        authenticated_header(request, responsible)
        get 'index'
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end
    describe 'GET /housing_offers as collaborateur' do
      before :each do
        authenticated_header(request, user5)
        get 'index'
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(3)
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq(3)
      end
    end
    describe 'GET /housing_offers/:id' do
      context 'as responsible' do
        before :each do
          authenticated_header(request, responsible)
          get 'show', id: housing_offer1.id
        end

        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end

        it 'returns valid JSON' do
          subject
          expect(json['data']['id']).to eq(housing_offer1.id.to_s)
          expect(json['data']['attributes']['lessor']['title']).to eq(lessor.title)
          expect(json['data']['attributes'].key?('reference')).to be_truthy
          expect(json['data']['attributes'].key?('release_date')).to be_truthy
        end
      end

      context 'with inaccessible housing_offer' do
        before :each do
          authenticated_header(request, responsible)
          get 'show', id: 'test_unavailable_housing_offer'
        end

        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end

    describe 'PUT /housing_offers/:id as a lessor' do
      describe 'update my housing_offer' do
        let!(:u_lessor) {create(:user, auth_roles: ['bailleur'])}
        let!(:ur) {create(:user_right, user: u_lessor, type: 'lessor', lessor_id: lessor.id, right: 'administrator')}
        
        let(:valid_params) do
          {
            id: housing_offer2.id.to_s,
            data: {attributes: {rent_amount: 245.62}}
          }
        end

        before :each do
          housing_offer2.update(lessor_siren: lessor.siren)
        end

        it 'updated form' do
          authenticated_header(request, u_lessor)
          put 'update', valid_params
          expect(response.status).to eq(200)
          expect(json['data']['id']).to eq(housing_offer2.id.to_s)
          expect(json['data']['attributes']['rent_amount']).to eq(245.62)
        end

        it 'throw error when publishing with no right' do 
          authenticated_header(request, u_lessor)
          put 'update', valid_params.merge(data: {attributes: {offer_status: "LOGEMENT PUBLIE"}})
          expect(response.status).to eq(401)
        end 

        it 'works when superadmin' do 
          authenticated_header(request, superadministrateur)
          put 'update', valid_params.merge(data: {attributes: {offer_status: "LOGEMENT PUBLIE"}})
          expect(response.status).to eq(200)
          expect(json['data']['attributes']['offer_status']).to eq("LOGEMENT PUBLIE")
        end 
      end
    end
  end

  context 'as bailleur' do
    let(:bailleur) {create :user, :lessor}
    let(:housing_offer1) {create(:housing_offer, user_id: bailleur.id)}
    let(:housing_offer2) {create(:housing_offer)}

    describe 'GET /housing_offers/:id/history' do
      before do
        create_list(:lessor_objective_history, 4, housing_offer_id: housing_offer1.id)
        create_list(:lessor_objective_history, 4, housing_offer_id: housing_offer2.id)
      end

      describe 'has access' do
        before do
          authenticated_header(request, bailleur)
          get 'history', id: housing_offer1.id
        end

        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq 4
        end
      end

      describe 'is forbidden to access' do
        before do
          authenticated_header(request, bailleur)
          get 'history', id: housing_offer2.id
        end

        it 'returns error' do
          expect(response.status).to eq(404)
        end
      end
    end
  end

  describe 'GET /housing_offers/with_recount_eligible_requests_nb' do 
    let!(:offer1) {create(:housing_offer, :with_eligible_requests, requests_count: 5)}
    let!(:e1) {create(:entity, siren: '000000001' )}
    let!(:e2) {create(:entity, siren: '000000002' )}
    let!(:e3) {create(:entity, siren: '000000003' )}
    let!(:rh) {create(:reservation_holder, siren: [e1.siren, e2.siren])}

    before do 
      authenticated_header(request, responsible)
    end 

    it 'with no housing_offer_ids' do 
      get 'with_recount_eligible_requests_nb'
      expect(response).to have_http_status(422)
      expect(json['errors'].length).to eq(1)
    end 

    it 'works for unelibile offer' do 
      offer1.set(eligibility_query_base: {typology_range: 'T111'}.to_json)
      get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s]
      expect(response).to have_http_status(200)
      expect(json['data'].first['attributes']['eligible_requests_nb']).to eq(0)
    end 
    
    it 'with no siren' do 
      get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s]
      expect(response).to have_http_status(200)
      expect(json['data'].first['attributes']['eligible_requests_nb']).to eq(5)
      expect(json['data'].first['attributes']['possible_eligible_requests_nb']).to eq(nil)
    end 

    it 'with siren' do 
      offer1.eligibles_requests.each_with_index do |hr, i |
        hr.set(entity_id: e1.id) if i == 1
        hr.set(entity_id: e2.id) if i == 2
        hr.set(entity_id: e3.id) if i == 3
      end 

      get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s], reserving_siren: e1.siren
      expect(response).to have_http_status(200)
      expect(json['data'].first['attributes']['eligible_requests_nb']).to eq(2)
    end 

    it 'for user not eligibile' do 
      authenticated_header(request, user)
      get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s]
      expect(response).to have_http_status(200)
      expect(json['data'].length).to eq(0)
    end 

    it 'with options possible_eligible_requests_nb' do 
      get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s], options: ['possible_eligible_requests_nb']
      expect(response).to have_http_status(200)
      expect(json['data'].first['attributes']['eligible_requests_nb']).to eq(5)
      expect(json['data'].first['attributes']['possible_eligible_requests_nb']).to eq(5)
    end 

    describe 'for user not eligible with application' do 
      let!(:ha) { create(:housing_application, housing_offer: offer1, housing_request: Hermes::HousingRequest.first,)}

      it 'works' do 
        authenticated_header(request, user)
        ha.set(user_id: user.id)
        get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s]
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(1)
        expect(json['data'].first['id']).to eq(offer1.id.to_s)
      end 

    end

  end 

  describe 'GET housing_offers/:id/eligible_requests' do 
    let!(:offer1) {create(:housing_offer, :with_eligible_requests, requests_count: 5)}

    it 'when sort by cotations' do 
      offer1.eligibles_requests.each_with_index do |hr, i|
        new_cotations =
          case i
          when 0
            {"34000" => 100}
          when 1
            {"34000" => 300}
          when 2
            {"34000" => 200}
          when 3
            {"34000" => 100}
          when 4
            nil
          end
        hr.update( cotations: new_cotations )
      end 

      authenticated_header(request, responsible)
      get 'eligible_requests', id: offer1.id, sort: { 'cotations': -1}
      expect(json['data'].length).to eq(5)
      expect(json['data'].map{ |d| d['attributes']['cotations'] && d['attributes']['cotations']['34000']}).to eq([300, 200, 100, 100, nil])
    end 
  end 

  describe 'GET /housing_offers/with_recount_eligible_requests_nb' do 
    let!(:offer1) {create(:housing_offer, :with_eligible_requests, requests_count: 5)}
    let!(:e1) {create(:entity, siren: '000000001' )}
    let!(:e2) {create(:entity, siren: '000000002' )}
    let!(:e3) {create(:entity, siren: '000000003' )}
    # TODO uncomment when merging multires
    # let!(:rh) {create(:reservation_holder, siren: [e1.siren, e2.siren])}

    before do 
      authenticated_header(request, responsible)
    end 

    it 'with no housing_offer_ids' do 
      get 'with_recount_eligible_requests_nb'
      expect(response).to have_http_status(422)
      expect(json['errors'].length).to eq(1)
    end 

    it 'with no siren' do 
      get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s]
      expect(response).to have_http_status(200)
      expect(json['data'].first['attributes']['eligible_requests_nb']).to eq(5)
    end 

    # TODO uncomment when merging multires
    # it 'with siren' do 
    #   offer1.eligibles_requests.each_with_index do |hr, i |
    #     hr.set(entity_id: e1.id) if i == 1
    #     hr.set(entity_id: e2.id) if i == 2
    #     hr.set(entity_id: e3.id) if i == 3
    #   end 

    #   get 'with_recount_eligible_requests_nb', housing_offer_ids: [offer1.id.to_s], reserving_siren: e1.siren
    #   expect(response).to have_http_status(200)
    #   expect(json['data'].first['attributes']['eligible_requests_nb']).to eq(2)
    # end
  end

  describe 'get housing_offers/:id with pictures' do 
    let!(:blob) { File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/')).read }
    (1..5).each do |i|
      let!("picture#{i}".to_sym) {create(:picture, thumb240_id: ImageTools.store_preview(blob, 240, 240))}
    end 
    let!(:offer) {create(:housing_offer, ordered_picture_ids: [picture1.id.to_s, picture2.id.to_s, picture3.id.to_s, picture4.id.to_s, picture5.id.to_s])}

    it 'show all pictures' do 
      authenticated_header(request, responsible)
      get 'show', id: offer.id
      attributes = json['data']['attributes']
      expect(attributes['pictures'].length).to eq(5)
      expect(attributes['main_picture']['thumb240']).not_to be_blank
    end 

    it 'show all picture_urls' do 
      offer.set(
        updated_at: DateTime.current,
        ordered_picture_ids: [],
        picture_urls: [
          { 'thumb240': 'http://localhost/test1.jpg'},
          { 'thumb240': 'http://localhost/test2.jpg'},
          { 'thumb240': 'http://localhost/test3.jpg'},
        ]
      ) 
      authenticated_header(request, responsible)
      get 'show', id: offer.id
      attributes = json['data']['attributes']
      expect(attributes['pictures'].length).to eq(3)
      expect(attributes['main_picture']['thumb240']).not_to be_blank
    end 

    describe 'default picture' do 

      before do 
        offer.set(updated_at: DateTime.current, ordered_picture_ids: [])
        authenticated_header(request, responsible)
      end 

      after do 
        HERMES_CONFIG['google_street_view'] = nil
      end 

      it 'when no config' do 
        get 'show', id: offer.id
        attributes = json['data']['attributes']
        expect(attributes['main_picture']['is_default']).to eq(true)
        expect(attributes['main_picture']['thumb240']).to include("maps.googleapis.com") 
      end
      
      it 'with config' do 
        HERMES_CONFIG['google_street_view'] = {
          key: 'google_key',
          cache_url: 'https://streetview.al-in.fr',
          signature: 'google_signature',
        }.stringify_keys
        get 'show', id: offer.id
        attributes = json['data']['attributes']
        expect(attributes['main_picture']['thumb240']).to include("streetview.al-in.fr") 
        params = CGI::parse(attributes['main_picture']['thumb240'])
        expect(params["key"].first).to eq("google_key") 
        expect(params["signature"].first).not_to eq('google_signature') 
        expect(params["signature"].first).not_to be_blank 
      end 
    end 
  end  

  describe 'rights_filtered routes' do 
    let!(:entity1) { create(:entity)}
    let!(:entity2) { create(:entity)}
    let!(:entity3) { create(:entity)}
    let!(:offer1) {create(:housing_offer, :with_eligible_requests, requests_count: 5)}
    let!(:eligible_requests) { offer1.eligibles_requests.to_a}
    (0..4).each do |i|
      let!("hr#{i+1}".to_sym) { eligible_requests[i] }
    end 
    let!(:user1) { create(:user, auth_roles: ['collaborateur offre logement'])}
    let!(:user2) { create(:user, auth_roles: ['entreprise'])}
    let!(:user_both) { create(:user, auth_roles: ['superadministrateur'])}
    let!(:user_none) { create(:user, auth_roles: ['collaborateur offre logement'])}
    let!(:user_right1) { create(:user_right, user: user1, entity_id: entity1.id, type: 'entity', right: 'administrator')}
    let!(:user_right2) { create(:user_right, user: user2, entity_id: entity2.id, type: 'entity', right: 'administrator')}
    let!(:user_right3) { create(:user_right, user: user_both, entity_id: entity1.id, type: 'entity', right: 'administrator')}
    let!(:user_right4) { create(:user_right, user: user_both, entity_id: entity2.id, type: 'entity', right: 'administrator')}
    let!(:entity_reservation1) { create(:entity_reservation, housing_offer: offer1, entity: entity1, from: Date.today - 10.days, to: Date.today + 10.days )} 
    let!(:entity_reservation2) { create(:entity_reservation, housing_offer: offer1, entity: entity2, from: Date.today - 10.days, to: Date.today + 10.days )} 
    
    before :each do 
      hr1.set(entity_id: entity1.id, entity_validated: true)
      hr2.set(entity_id: entity1.id, entity_validated: true)
      hr3.set(entity_id: entity2.id, entity_validated: true)
      hr4.set(entity_id: entity3.id, entity_validated: true)
    end 
    
    describe 'with_recount_eligible_requests_nb_rights_filtered' do 
      let!(:ha1) { create(:housing_application, housing_request: hr1, housing_offer: offer1, status: 'EN ATTENTE')}
      let!(:ha2) { create(:housing_application, housing_request: hr3, housing_offer: offer1, status: 'EN ATTENTE')}

      it 'housing_offer_ids_params_is_missing' do 
        authenticated_header(request, user1)
        post 'with_recount_eligible_requests_nb_rights_filtered'
        expect(response).to have_http_status(422)
      end 
    
      it 'for partial user' do 
        authenticated_header(request, user1)
        post 'with_recount_eligible_requests_nb_rights_filtered', housing_offer_ids: [offer1.id]
        expect(response).to have_http_status(200)
        attributes = json['data'].first['attributes']
        expect(attributes['eligible_requests_nb']).to eq(2)
        expect(attributes['possible_eligible_requests_nb']).to eq(nil)
        expect(attributes['applicated_nb']).to eq(1)
      end 

      it 'for full user' do 
        authenticated_header(request, user_both)
        post 'with_recount_eligible_requests_nb_rights_filtered', housing_offer_ids: [offer1.id]
        expect(response).to have_http_status(200)
        attributes = json['data'].first['attributes']
        expect(attributes['eligible_requests_nb']).to eq(3)
        expect(attributes['applicated_nb']).to eq(2)
      end 

      it 'for no right user' do 
        authenticated_header(request, user_none)
        post 'with_recount_eligible_requests_nb_rights_filtered', housing_offer_ids: [offer1.id]
        expect(response).to have_http_status(200)
        attributes = json['data'].first['attributes']
        expect(attributes['eligible_requests_nb']).to eq(0)
        expect(attributes['applicated_nb']).to eq(0)
      end 

      it 'with entity_ids filter' do 
        authenticated_header(request, user_both)
        post 'with_recount_eligible_requests_nb_rights_filtered', housing_offer_ids: [offer1.id], entity_ids: [entity1.id]
        expect(response).to have_http_status(200)
        attributes = json['data'].first['attributes']
        expect(attributes['eligible_requests_nb']).to eq(2)
        expect(attributes['applicated_nb']).to eq(1)
      end 

      it 'with possible_eligible_requests_nb options' do 
        authenticated_header(request, user_both)
        post 'with_recount_eligible_requests_nb_rights_filtered', housing_offer_ids: [offer1.id], options: ['possible_eligible_requests_nb']
        expect(response).to have_http_status(200)
        attributes = json['data'].first['attributes']
        expect(attributes['eligible_requests_nb']).to eq(3)
        expect(attributes['possible_eligible_requests_nb']).to eq(3)
      end 
    end 

    describe 'eligible_requests_rights_filtered' do
      it 'for partial user' do 
        authenticated_header(request, user1)
        post 'eligible_requests_rights_filtered', id: offer1.id
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(2)
      end 

      it 'for full user' do 
        authenticated_header(request, user_both)
        post 'eligible_requests_rights_filtered', id: offer1.id
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(3)
      end 

      it 'for no right user' do 
        authenticated_header(request, user_none)
        post 'eligible_requests_rights_filtered', id: offer1.id
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(0)
      end 

      it 'cannot manage' do 
        user1.set(auth_roles: ['demandeur'], updated_at: DateTime.current)
        authenticated_header(request, user1)
        post 'eligible_requests_rights_filtered', id: offer1.id
        expect(response).to have_http_status(401)
      end 

      it 'works with filter' do 
        offer1.eligibles_requests.order_by(created_at: :desc).first.set(housing_channel_keys: ['dc', 'dalo']) 
        offer1.eligibles_requests.order_by(created_at: :desc).last.set(housing_channel_keys: ['test']) 
        
        authenticated_header(request, user_both)
        get 'eligible_requests_rights_filtered', id: offer1.id
        expect(json['data'].length).to eq(2)
        get 'eligible_requests_rights_filtered', id: offer1.id, housing_channel_keys: 'dalo'
        expect(json['data'].length).to eq(1)
        get 'eligible_requests_rights_filtered', id: offer1.id, housing_channel_keys: 'test'
        expect(json['data'].length).to eq(0)
      end 

    end 

    describe 'all_applications_rights_filtered' do
      let!(:ha1) { create(:housing_application, housing_request: hr1, housing_offer: offer1, status: 'EN ATTENTE')}
      let!(:ha2) { create(:housing_application, housing_request: hr3, housing_offer: offer1, status: 'EN ATTENTE')}

      it 'for partial user' do 
        authenticated_header(request, user1)
        post 'all_applications_rights_filtered', id: offer1.id
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(1)
        expect(json['data'].first['id']).to eq(ha1.id.to_s)
      end 

      it 'for full user' do 
        authenticated_header(request, user_both)
        post 'all_applications_rights_filtered', id: offer1.id
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(2)
        expect(json['data'].first['id']).to eq(ha1.id.to_s)
        expect(json['data'].last['id']).to eq(ha2.id.to_s)
      end 

      it 'for no right user' do 
        authenticated_header(request, user_none)
        post 'all_applications_rights_filtered', id: offer1.id
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(0)
      end 

      it 'cannot manage' do 
        user1.set(auth_roles: ['demandeur'], updated_at: DateTime.current)
        authenticated_header(request, user1)
        post 'all_applications_rights_filtered', id: offer1.id
        expect(response).to have_http_status(404)
      end
      
      it 'with entity_id filter' do 
        authenticated_header(request, user_both)
        post 'all_applications_rights_filtered', id: offer1.id, entity_ids: [hr1.entity_id]
        expect(response).to have_http_status(200)
        expect(json['data'].length).to eq(1)
        expect(json['data'].first['id']).to eq(ha1.id.to_s)
      end 

    end 
  end
  
  describe 'GET /housing_offers/simulate_eligible_requests_count' do 
    let!(:threshold_plus) do
      create(
        :threshold,
        type: "PLUS",
        codes: ["PLU"],
        thresholds_map: {
          "Paris et communes limitrophes"=>{"thresholds_list"=>[23721, 35452, 46473, 55486, 66017, 74286], "another_person"=>8278},
          "IDF hors Paris et Communes limitrophes"=>{"thresholds_list"=>[23721, 35452, 42616, 51046, 60429, 68001], "another_person"=>7577},
          "Autres régions"=>{"thresholds_list"=>[20623, 27540, 33119, 39982, 47035, 53008], "another_person"=>5912}
        }
      )
    end

    let!(:offer) {create(:housing_offer, :with_eligible_requests, requests_count: 6)}
    let!(:params) do 
      { 
        threshold_type: 'PLUS',
        threshold_subtype: "Paris et communes limitrophes",
        rent_with_charges: 600,
        insee_code: offer.insee_code,
        typology: offer.typology,
      }
    end

    before do 
      authenticated_header(request, responsible)
      offer.eligibles_requests.each_with_index do |hr, i|
        hr.set(request_status: 'EN ATTENTE DE PIECES')  if i == 0
        hr.set(request_status: 'ANNULEE')  if i == 1
        hr.set(is_active: false) if i == 2
        hr.set(typology_range: ['T6'], max_typology: 6, min_typology: 6)  if i == 3
        hr.set(bordering_insee_codes: ['34000'], seeked_district_insee_codes: nil)  if i == 4
      end 
    end 

    it 'missing params' do 
      get 'simulate_eligible_requests_count', params.delete(:insee_code)
      expect(response).to have_http_status(422)
    end 

    it 'works' do 
      get 'simulate_eligible_requests_count', params
      hr = offer.eligibles_requests.last

      expect(response).to have_http_status(200)
      expect(json['complete_eligible_requests_count']).to eq(2)
      expect(json['possible_eligible_requests_count']).to eq(3)
    end 

    it 'works with ,' do 
      params[:rent_with_charges] = '600,00'
      get 'simulate_eligible_requests_count', params
      hr = offer.eligibles_requests.last

      expect(response).to have_http_status(200)
      expect(json['complete_eligible_requests_count']).to eq(2)
      expect(json['possible_eligible_requests_count']).to eq(3)
    end 

  end
end 

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::HousingOffersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_HOUSING_OFFERS'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
  let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
  let!(:applicant) {create(:applicant, user: responsible)}
  let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s, als_id: '123456')}
  let!(:housing_offer) {create(:housing_offer, external_ref: '123456')}

  let!(:als_rh) {Hermes::ReservationHolder.create_default}
  let!(:admin) {create(:administrator)}
  let!(:bailleur) { create :user, roles: ['bailleur'] }
  let!(:housing_channel) { create :housing_channel, key: 'dalo' }
  let!(:lessor) { create :lessor, user_id: bailleur.id }

  let!(:threshold1) do
    create(
      :threshold,
      type: 'PLA LM',
      thresholds_map: {
        'Paris et communes limitrophes': {
          'thresholds_list': [
            13_050,
            21_272,
            27_883,
            30_521,
            36_307,
            40_859
          ],
          'another_person': 4552
        },
        'IDF hors Paris et Communes limitrophes': {
          'thresholds_list': [
            13_050,
            21_272,
            25_569,
            28_075,
            33_238,
            37_401
          ],
          'another_person': 4166
        },
        'Autres régions': {
          'thresholds_list': [
            11_342,
            16_525,
            19_872,
            22_111,
            25_870,
            29_155
          ],
          'another_person': 3252
        }
      }
    )
  end

  ['index', 'search'].each do |type|
    path "/api/dmo/housing_offers#{type == 'search' ? '/search' : ''}" do
      method = type == 'search' ? :post : :get
      send(method, 'Retrieves all housing_offers') do
        tags tags
        produces 'application/json'
        Hermes::Api::Dmo::HousingOffersController.acceptable_params.sort.each do |params_name, options|
          parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
        end
        parameter name: 'postal_codes', in: :query, type: :array, required: false

        security [Bearer: {}]
        paginator

        response '200', 'housing_offers found' do
          let(:Authorization) {"Bearer #{token_for(admin)}"}
          let!(:housing_offers) {create_list(:housing_offer, 5)}

          include_context 'with integration test'
          include_context 'valid jsonapi collection' do
            let(:expected_collection) {housing_offers + [housing_offer]}
          end
        end
      end
    end
  end 

  path '/api/dmo/housing_offers/{id}' do
    get 'Retrieves a single housing_offer by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      let(:housing_offer) {create(:housing_offer)}
      let(:id) {housing_offer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
        run_test!
      end 

      response '200', 'housing_offer found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}

        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {housing_offer}
        end
      end
    end
  end

  path '/api/dmo/housing_offers' do
    post 'Create housing_offer' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::HousingOffersController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )

      let(:params) do
        jsonapi_attributes(
          {external_ref: 'TEST'}
        )
      end

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '201', 'housing_offer created' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'

        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'housing_offers'
          expect(json['data']['attributes']['external_ref']).to eq 'TEST'
        end
      end
    end
  end

  path '/api/dmo/housing_offers/{id}' do
    put 'edit housing_offer' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::HousingOffersController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )

      let(:params) do
        jsonapi_attributes(
          {external_ref: 'TEST'}
        )
      end
      let(:housing_offer) {create(:housing_offer)}
      let(:id) {housing_offer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
        run_test!
      end 

      response '200', 'housing_offers updated' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'
        after do |_example|
          json = JSON.parse(response.body)

          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'housing_offers'
        end
      end
    end
  end

  path '/api/dmo/housing_offers/{id}' do
    delete 'Delete a single housing_offer by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      let(:id) {create(:housing_offer, user_id: responsible.id).id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
        run_test!
      end 


      response '204', 'housing_offer deleted' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/with_validated_applications/{id}' do
    get 'Get validated application for an offer' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      parameter name: :from, in: :query, type: :string

      security [Bearer: {}]
      let(:id) { housing_offer.id}
      let(:from) {Date.today - 1.days}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
        run_test!
      end 

      response '200', 'applications found' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        let!(:housing_applications) {create_list(:housing_application, 5, status: 'TRANSMIS BAILLEUR', housing_offer_id: housing_offer.id, housing_request_id: housing_request.id)}

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) {housing_applications}
        end
      end
    end
  end

  path '/api/dmo/housing_offers/reference_exist' do
    get 'Check if an active offer exist for an external_ref' do
      tags tags
      produces 'application/json'
      parameter name: :external_ref, in: :query, type: :string, required: true
      let(:external_ref) { housing_offer.external_ref}
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'housing_offer found' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/with_recount_eligible_requests_nb' do
    get 'Get offers with a recount of eligible_requests_nb' do
      tags tags
      produces 'application/json'
      parameter name: :housing_offer_ids, in: :query, type: :array, required: true
      let(:housing_offer_ids) { [housing_offer.id]}
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'housing_offers found' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/with_recount_eligible_requests_nb_rights_filtered' do
    post 'Get offers with a recount of eligible_requests_nb' do
      tags tags
      produces 'application/json'
      parameter name: :housing_offer_ids, in: :query, type: :array, required: true
      let(:housing_offer_ids) { [housing_offer.id.to_s]}
      security [Bearer: {}]

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'housing_offers found' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/resend_flux5_all_missing_offers' do
    post 'Send all flux5 for all offer not succesfully sent' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      parameter name: :date, in: :query, type: :string, required: true
      let(:date) {Date.today - 1.days}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'housing_offer created' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!

        after do |_example|
          json = JSON.parse(response.body)
          expect(json['success']).to eq true
        end
      end
    end
  end

  path '/api/dmo/housing_offers/simulate_eligible_requests_count' do
    get 'Simulate a offer and count its eligibles requests' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      parameter name: :threshold_type, in: :query, type: :string, required: true
      parameter name: :threshold_subtype, in: :query, type: :string, required: true
      parameter name: :rent_with_charges, in: :query, type: :integer, required: true
      parameter name: :insee_code, in: :query, type: :string, required: true
      parameter name: :typology, in: :query, type: :string, required: true

      let(:threshold_type) { threshold1.type }
      let(:threshold_subtype) { 'Paris et communes limitrophes' }
      let(:rent_with_charges) { 1000 }
      let(:insee_code) { '33000' }
      let(:typology) { 'T2' }

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'housing_offer created' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!

        after do |_example|
          json = JSON.parse(response.body)
          expect(json['complete_eligible_requests_count']).to eq 0
          expect(json['complete_eligible_requests_count']).to eq 0
        end
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/eligible_requests' do
    get 'Retrieves eligible_requests relative to a given housing_offer' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      Hermes::Api::Dmo::HousingRequestsController.acceptable_params.each do |acceptable_param, options|
        parameter name: acceptable_param.to_s, in: :query, required: false, type: options[:type]
      end
      paginator
      let(:id) { housing_offer.id }

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'eligible_requests found' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        include_context 'not containings errors'
        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/eligible_requests_rights_filtered' do
    post 'Retrieves eligible_requests relative to a given housing_offer filtered by accesible by user' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      Hermes::Api::Dmo::HousingRequestsController.acceptable_params.each do |acceptable_param, options|
        parameter name: acceptable_param.to_s, in: :query, required: false, type: options[:type]
      end
      paginator
      let(:id) { housing_offer.id }

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'eligible_requests found' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        include_context 'not containings errors'
        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/entity_statuses' do
    get 'Retrieves entity_statuses relative to a given housing_offer' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      parameter name: :entity_id, in: :query, type: :string, required: true
      security [Bearer: {}]

      let(:id) { housing_offer.id }
      let(:entity) { create(:entity, readers: [responsible.id.to_s])}
      let(:entity_id) { entity.id }

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'statuses found' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        include_context 'not containings errors'
        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/selected_applications' do
    get 'Retrieves selected housing_applications relative to a given housing_offer' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      Hermes::Api::Dmo::HousingApplicationsController.acceptable_params.each do |acceptable_param, options|
        parameter name: acceptable_param.to_s, in: :query, type: options[:type], required: false
      end
      
      paginator
      let(:id) {housing_offer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'application found' do
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        let!(:housing_requests) do
          create_list(:housing_request, 5, main_applicant_id: applicant.id.to_s) do |request, i|
            create(:housing_application,
                    housing_offer_id: housing_offer.id.to_s,
                    housing_request_id: request.id.to_s,
                    status: 'PLO'
            )
            create(:housing_application,
                    housing_offer_id: housing_offer.id.to_s,
                    housing_request_id: request.id.to_s,
                    status: 'TRANSMIS BAILLEUR'
            )
          end
        end

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/selected_applications' do
    post 'Select housing application for a offer' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params({ housing_application_ids: { type: :array, items: { type: :string } }}),
      )

      let(:params) do
        jsonapi_attributes(
          {housing_application_ids: [housing_application.id.to_s]}
        )
      end
      let(:id) {housing_offer.id}
      let(:housing_request) {create(:active_request)}
      let(:housing_application) {create(:housing_application, housing_offer_id: housing_offer.id.to_s, housing_request_id: housing_request.id.to_s, status: 'EN ATTENTE')}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'housing_application selected' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        let!(:housing_requests) do
          create_list(:housing_request, 5, main_applicant_id: applicant.id.to_s, user: responsible) do |request, i|
            create(:housing_application,
                    housing_offer_id: housing_offer.id.to_s,
                    housing_request_id: request.id.to_s,
                    status: 'PLO'
            )
            create(:housing_application,
                    housing_offer_id: housing_offer.id.to_s,
                    housing_request_id: request.id.to_s,
                    status: 'TRANSMIS BAILLEUR'
            )
          end
        end

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/all_applications' do
    get 'Get all application from an offer' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      Hermes::Api::Dmo::HousingApplicationsController.acceptable_params.each do |acceptable_param, options|
        parameter name: acceptable_param.to_s, in: :query, type: options[:type], required: false
      end
      
      paginator
      let(:id) {housing_offer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'applications found' do
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/all_applications_rights_filtered' do
    post 'Get all application from an offer filtered by accessible by user' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      Hermes::Api::Dmo::HousingApplicationsController.acceptable_params.each do |acceptable_param, options|
        parameter name: acceptable_param.to_s, in: :query, type: options[:type], required: false
      end
      
      paginator
      let(:id) {housing_offer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'applications found' do
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/history' do
    get 'Get all lessor objective history of an offer' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      paginator
      let(:id) {housing_offer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'history found' do
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/add_housing_channel' do
    post 'Add an housing channel' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      parameter name: :key, in: :query, type: :string, required: true
      security [Bearer: {}]
      let(:key) {housing_channel.key}

      paginator
      let(:id) {housing_offer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'added housing channel' do
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/remove_housing_channel' do
    post 'Remove an housing channel' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      parameter name: :key, in: :query, type: :string, required: true
      security [Bearer: {}]
      let(:key) {housing_channel.key}

      paginator
      let(:id) {housing_offer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'removed housing channel' do
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/resend_flux5' do
    post 'Resend flux 5 for an offer' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]

      paginator
      let(:id) {housing_offer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'flux sent' do
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end


  path '/api/dmo/housing_offers/{id}/is_eligible_request/{als_id}' do
    get 'Check if a request is eligible to an offer' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      parameter name: :als_id, in: :path, type: :string

      security [Bearer: {}]

      paginator
      let(:id) {housing_offer.id}
      let(:als_id) { '123456'}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '200', 'is eliglibe' do
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}

        run_test!
      end
    end
  end
end
