require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::PicturesController, :type => :api do
  
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  
  let!(:responsible) { create(:user, roles: ['Administrator']) }
  let!(:cat_picture) do 
    Rack::Test::UploadedFile.new(File.join(Rails.root, 'packages', 'hermes', 'spec', 'fixtures', 'files', 'cat1.jpg'), 'image/jpeg')
  end
  let!(:pdf_as_image) do 
    Rack::Test::UploadedFile.new(File.join(Rails.root, 'packages', 'hermes', 'spec', 'fixtures', 'files', 'pdf_as_image.jpg'), 'image/jpeg')
  end

  shared_examples "successful picture upload" do
    it 'works' do
      subject
      expect(response).to have_http_status(201)
    end
  end
  
  shared_examples "no file upload" do
    it 'when no file' do
      post action, params.except(:file)
      expect(response).to have_http_status(422)
      expect(response.body).to include('no_file_in_the_request')
    end
  end
  
  shared_examples "unsupported media type" do
    it 'when unsupported media type' do
      post action, params.merge(file: pdf_as_image)
      expect(response).to have_http_status(415)
      expect(response.body).to include('not_an_image')
    end

    it 'when wrong format' do
      post action, params.merge(file: "wrong_format")
      expect(response).to have_http_status(415)
      expect(response.body).to include('not_an_image')
    end
  end

  describe 'POST /pictures' do
    let!(:params) { { file: cat_picture} }
    let(:action) { 'create'}

    before :each do
      authenticated_header(request, responsible)
    end 

    subject { post 'create', params }

    it_behaves_like "successful picture upload"
    it_behaves_like "no file upload"
    it_behaves_like "unsupported media type"

    %w[
      rdsl
      lessor
      collaborateur offer logement
    ].each do |role|
      context "when #{role}" do
        let!(:user) {create(:user, auth_roles: [role])}
        it 'works' do
          authenticated_header(request, user)
          subject
          expect(response).to have_http_status(201)
        end
      end
    end 

    it 'cannot upload' do 
      user = create(:user)
      authenticated_header(request, user)
      subject
      expect(response).to have_http_status(401)
      expect(response.body).to include('cannot_upload_free_picture')
    end 
  end

  describe 'POST /housing_offers/:id/pictures' do
    let!(:housing_offer) {create(:housing_offer, user_id: responsible.id)}
    let!(:params) { { file: cat_picture, id: housing_offer.id} }
    let(:action) { 'add_picture_to_offer' }

    before :each do
      authenticated_header(request, responsible)
    end 

    subject { post 'add_picture_to_offer', params }

    it 'works' do
      subject
      expect(response).to have_http_status(201)
      picture = Hermes::Picture.find(json['data']['id'])
      expect(housing_offer.reload.ordered_picture_ids).to include(picture.id)
      expect(picture.reload.housing_offer_id).to eq(housing_offer.id)
    end

    it 'when housing_offer_not_found' do 
      post 'add_picture_to_offer', params.merge(id: 'test')
      expect(response).to have_http_status(404)
      expect(response.body).to include('housing_offer_not_found')
    end 

    it_behaves_like "no file upload"
    it_behaves_like "unsupported media type"

    it 'cannot upload' do 
      user = create(:user)
      authenticated_header(request, user)
      subject
      expect(response).to have_http_status(401)
      expect(response.body).to include('cannot_upload_to_offer')
    end 

  end 

  shared_examples 'successful deletion' do
    it 'works' do
      authenticated_header(request, responsible)
      subject
      expect(response).to have_http_status(204)
    end
  end
  
  shared_examples 'no access' do |user_factory|
    before do
      user = create(user_factory || :user)
      authenticated_header(request, user)
    end
  
    it 'returns 401 status' do
      subject
      expect(response).to have_http_status(401)
    end
  end
  
  describe 'DELETE /pictures/{:id}' do
    let!(:user) {create(:user)}
    let!(:picture) {create(:picture)}
    let!(:id) { picture.id.to_s }
    subject { delete 'destroy', id: id }

    before do
      authenticated_header(request, responsible)
    end 

    include_examples 'successful deletion'
    include_examples 'no access', :user

    context 'when no picture found' do
      let!(:id) { 'test' }
      before {subject }

      it {expect(response).to have_http_status(404)}
    end

    it 'when access with offer' do
      offer = create(:housing_offer, readers: [user.id.to_s], ordered_picture_ids: [picture.id.to_s])
      authenticated_header(request, user)
      subject
      expect(response).to have_http_status(204)
      expect(offer.reload.ordered_picture_ids).to_not include(picture.id.to_s)
    end

  end 

  describe 'DELETE /housing_offers/:id/pictures/:picture_id' do
    let!(:offer) {create(:housing_offer)}
    let!(:picture) {create(:picture, housing_offer: offer)}
    let!(:picture_id) { picture.id.to_s }
    let!(:id) { offer.id.to_s }
    let!(:user) {create(:user)}

    subject { delete 'delete_picture_from_offer', id: id, picture_id: picture_id }

    before :each do
      authenticated_header(request, responsible)
    end 

    include_examples 'successful deletion'
    include_examples 'no access', :user

    context 'when no housing_offer found' do
      let!(:id) { 'test' }
      before {subject }

      it {expect(response).to have_http_status(404)}
    end

    context 'when no picture found' do
      let!(:picture_id) { 'test' }
      before {subject }

      it {expect(response).to have_http_status(404)}
    end

  end 
end

describe Hermes::Api::Dmo::PicturesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  TAG = 'API_DMO_PICTURES'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
  let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

  # Pictures functionalities
  path '/api/dmo/pictures' do
    post 'create_picture' do
      tags TAG
      produces 'application/json'
      parameter name: 'Content-Type', in: :header, type: :string, required: false, description: "must be 'multipart/form-data'"
      consumes 'multipart/form-data'
      # consumes 'application/json'
      security [Bearer: {}]
      parameter name: :file, in: :formData, type: :file, description: 'picture to upload'
      description 'Send a file with the post action to add a picture'

      response '201', 'picture created' do
        let(:file) do
          Rack::Test::UploadedFile.new([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'), 'mime/type')
        end
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/pictures' do
    post 'create_picture on offer' do
      tags TAG
      produces 'application/json'
      parameter name: 'Content-Type', in: :header, type: :string, required: false, description: "must be 'multipart/form-data'"
      consumes 'multipart/form-data'
      # consumes 'application/json'
      security [Bearer: {}]
      parameter name: :id, in: :path, type: :file, description: 'id of the housing_offer'
      parameter name: :file, in: :formData, type: :file, description: 'picture to upload'

      description 'Send a file with the post action to add a picture to the offer'

      response '201', 'picture created' do
        let!(:id) {create(:housing_offer, user_id: responsible.id).id.to_s}
        let(:file) do
          Rack::Test::UploadedFile.new([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'), 'mime/type')
        end
        let(:'Content-Type') {'multipart/form-data'}
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'
      end
    end
  end

  path '/api/dmo/pictures/{id}' do
    delete 'delete_picture' do
      tags TAG
      produces 'application/json'
      consumes 'application/json'
      parameter name: :id, in: :path, type: :string, required: true, description: 'ID of the picture'
      security [Bearer: {}]

      response '204', 'picture deleted' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        let!(:id) {create(:picture, user_id: responsible.id).id.to_s}

        run_test!
      end
    end
  end

  path '/api/dmo/housing_offers/{id}/pictures/{picture_id}' do
    delete 'delete_picture on a offer' do
      tags TAG
      produces 'application/json'
      consumes 'application/json'
      parameter name: :picture_id, in: :path, type: :string, required: true, description: 'ID of the picture'
      parameter name: :id, in: :path, type: :string, required: true, description: 'ID of the offer'

      security [Bearer: {}]

      response '204', 'picture deleted' do
        let(:Authorization) {"Bearer #{token_for(responsible)}"}
        let!(:offer) {create(:housing_offer, user_id: responsible.id)}
        let!(:id) {offer.id.to_s}
        let!(:picture_id) {create(:picture, housing_offer: offer).id.to_s}

        run_test!
      end
    end
  end
end