require 'rails_helper'
require [Hermes::Engine.root, %w[app lib notifications_center user]].flatten.join('/').to_s

def perform_job_now(job)
  job[:job].perform_now(*deserialize_argument(job[:args]))
end 

RSpec.describe NotificationsCenter::User, type: :helper do
  let!(:user) { create(:user, email: 'test1@test.com') }
  let!(:entity) {create(:entity, siret: '12345678901234', title: 'Compagny')}
  let!(:applicant) { create(:applicant, user: user, main_employer_siret: entity.siret) }
  let!(:hr) { create(:housing_request, is_active: true, main_user_id: user.id, main_applicant: applicant) }
  let!(:offer1) {create(:housing_offer, insee_code: '75000', surface: 56, typology: 'T2', rent_with_charges: 840)}
  let!(:offer2) {create(:housing_offer, insee_code: '75000', surface: 32, typology: 'T1', rent_with_charges: 500)}

  before do 
    create_mail_template_default()
  end 

  # TODO : Refacto to not duplicate code
  it 'new_reserved_offer' do
    expect(hr.entity_id).to eq(entity.id)

    # Launch notification
    user.notify(
      key: NOTIFICATION_KEYS[:user][:new_reserved_offer],
      channels: ['email'],
      datas: {offer_ids: [offer1.id.to_s, offer2.id.to_s]}
    )

    # select notification job 
    last_notif_job = enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.last
    expect(last_notif_job[:args][0]['key']).to eq('user.new_reserved_offer')
    
    # perform notification job and check on notif model created
    perform_job_now(last_notif_job)
    notif = ::NotificationCenter::Notification.last
    expect(notif.key).to eq('user.new_reserved_offer')
    
    # send notification
    notif.send_notification
    last_email_job = enqueued_jobs.select{|job| job[:job] == ActionMailer::DeliveryJob }.last
    perform_job_now(last_email_job)

    # test content notification
    contents = ::NotificationsCenter::User.new(instance: notif)
    expect(contents.title).to eq("Action Logement Services AL'In Logement exclusif Compagny")
    expect(contents.body).to include(offer1.id.to_s, offer2.id.to_s)
  end

  it 'new_published_offer' do 
    # Launch notification
    user.notify(
      key: NOTIFICATION_KEYS[:user][:new_published_offer],
      channels: ['email'],
      datas: {offer_ids: [offer1.id.to_s, offer2.id.to_s]}
    )

    # select notification job 
    last_notif_job = enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.last
    expect(last_notif_job[:args][0]['key']).to eq('user.new_published_offer')
    
    # perform notification job and check on notif model created
    perform_job_now(last_notif_job)
    notif = ::NotificationCenter::Notification.last
    expect(notif.key).to eq('user.new_published_offer')
    
    # test content notification
    contents = ::NotificationsCenter::User.new(instance: notif)
    expect(contents.title).to eq("Action Logement AL'in_De nouvelles offres sont disponibles")
    expect(contents.body).to include(offer1.id.to_s, offer2.id.to_s)
  end 
  it 'application_refusal' do 
    # Launch notification
    user.notify(
      key: NOTIFICATION_KEYS[:user][:application_refusal],
      channels: ['email'],
      datas: {   
        first_name: "martin",
        last_name: "matin" , 
        als_id: "33",
        city: "Ales",
      offer_ids: [offer1.id.to_s, offer2.id.to_s]}
    )

    # select notification job 
    last_notif_job = enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.last
    expect(last_notif_job[:args][0]['key']).to eq('user.application_refusal')
    
    # perform notification job and check on notif model created
    perform_job_now(last_notif_job)
    notif = ::NotificationCenter::Notification.last
    expect(notif.key).to eq('user.application_refusal')
    
    # test content notification
    contents = ::NotificationsCenter::User.new(instance: notif)
    expect(contents.title).to eq("Action Logement AL’in_Information sur votre candidature")
    expect(contents.body).to include(offer1.id.to_s, offer2.id.to_s)
  end 
  it 'application_offer_taken_back' do 
    # Launch notification
    user.notify(
      key: NOTIFICATION_KEYS[:user][:application_offer_taken_back],
      channels: ['email'],
      datas: {   
        first_name: "martin",
        last_name: "matin" , 
        als_id: "33",
        city: "Ales",
       applicated_at:  DateTime.new(2007,11,19,8,37,48,"-06:00").strftime("%d/%m/%Y"),
      offer_ids: [offer1.id.to_s, offer2.id.to_s]}
    )

    # select notification job 
    last_notif_job = enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.last
    expect(last_notif_job[:args][0]['key']).to eq('user.application_offer_taken_back')
    
    # perform notification job and check on notif model created
    perform_job_now(last_notif_job)
    notif = ::NotificationCenter::Notification.last
    expect(notif.key).to eq('user.application_offer_taken_back')
    
    # test content notification
    contents = ::NotificationsCenter::User.new(instance: notif)
    expect(contents.title).to eq("Action Logement AL’in_Information sur votre candidature")
    expect(contents.body).to include(offer1.id.to_s, offer2.id.to_s)
  end 
end
